# pylint: skip-file
# type: ignore

import typing as _tp

from PyQt5.QtWidgets import QGraphicsTextItem

import trnsysGUI.images as _img
from trnsysGUI.BlockItem import BlockItem
from trnsysGUI.PortItem import PortItem


class TVentil(BlockItem):
    def __init__(self, trnsysType, parent, **kwargs):
        super(TVentil, self).__init__(trnsysType, parent, **kwargs)

        self.h = 40
        self.w = 40
        self.typeNumber = 3
        self.isTempering = False
        self.positionForMassFlowSolver = 1.0
        self.posLabel = QGraphicsTextItem(str(self.positionForMassFlowSolver), self)
        self.posLabel.setVisible(False)

        self.exportInitialInput = 0.0

        self.inputs.append(PortItem("o", 0, self))
        self.inputs.append(PortItem("o", 1, self))
        self.outputs.append(PortItem("i", 2, self))

        self.changeSize()

    def _getImageAccessor(self) -> _tp.Optional[_img.ImageAccessor]:
        return _img.T_VENTIL_SVG

    def changeSize(self):
        w = self.w
        h = self.h

        delta = 20
        # Limit the block size:
        if h < 20:
            h = 20
        if w < 40:
            w = 40

        # center label:
        rect = self.label.boundingRect()
        lw, lh = rect.width(), rect.height()
        lx = (w - lw) / 2

        deltaH = self.h / 18

        self.label.setPos(lx, h - self.flippedV * (h + h / 2))
        self.posLabel.setPos(lx + 5, -15)

        self.origInputsPos = [[0, delta], [delta, 0]]
        self.origOutputsPos = [[w, delta]]
        self.inputs[0].setPos(self.origInputsPos[0][0], self.origInputsPos[0][1])
        self.inputs[1].setPos(self.origInputsPos[1][0], self.origInputsPos[1][1])
        self.outputs[0].setPos(self.origOutputsPos[0][0], self.origOutputsPos[0][1])

        self.updateFlipStateH(self.flippedH)
        self.updateFlipStateV(self.flippedV)

        self.inputs[0].side = (self.rotationN + 2 * self.flippedH) % 4
        self.inputs[1].side = (self.rotationN + 1 + 2 * self.flippedV) % 4
        self.outputs[0].side = (self.rotationN + 2 - 2 * self.flippedH) % 4

        return w, h

    def setComplexDiv(self, b):
        self.isTempering = bool(b)

    def setPositionForMassFlowSolver(self, f):
        self.positionForMassFlowSolver = f

    def encode(self):
        dictName, dct = super(TVentil, self).encode()
        dct["IsTempering"] = self.isTempering
        dct["PositionForMassFlowSolver"] = self.positionForMassFlowSolver
        return dictName, dct

    def decode(self, i, resBlockList):
        super().decode(i, resBlockList)
        if "IsTempering" not in i or "PositionForMassFlowSolver" not in i:
            self.logger.debug("Old version of diagram")
            self.positionForMassFlowSolver = 1.0
        else:
            self.isTempering = i["IsTempering"]
            self.positionForMassFlowSolver = i["PositionForMassFlowSolver"]

    def decodePaste(self, i, offset_x, offset_y, resConnList, resBlockList, **kwargs):
        super(TVentil, self).decodePaste(i, offset_x, offset_y, resConnList, resBlockList, **kwargs)
        if "IsTempering" or "PositionForMassFlowSolver" not in i:
            self.logger.debug("Old version of diagram")
            self.positionForMassFlowSolver = 1.0
        else:
            self.isTempering = i["IsTempering"]
            self.positionForMassFlowSolver = i["PositionForMassFlowSolver"]

    def exportParameterSolver(self, descConnLength):
        temp = ""

        for o in self.outputs:
            # ConnectionList lenght should be max offset
            for c in o.connectionList:
                if type(c.fromPort.parent, "heatExchangers") and o.connectionList.index(c) == 0:
                    continue
                elif type(c.toPort.parent, "heatExchangers") and o.connectionList.index(c) == 0:
                    continue
                else:
                    temp = temp + str(c.trnsysId) + " "
                    self.trnsysConn.append(c)

        for i in self.inputs:
            # Either left or right
            tempArr = []

            for c in i.connectionList:
                if type(c.fromPort.parent) and i.connectionList.index(c) == 0:
                    continue
                elif type(c.toPort.parent) and i.connectionList.index(c) == 0:
                    continue
                else:
                    # On same hight as output
                    if i.pos().x() == self.outputs[0].pos().x() or i.pos().y() == self.outputs[0].y():
                        tempArr.insert(0, c)
                    else:
                        tempArr.append(c)

            for conn in tempArr:
                temp = temp + str(conn.trnsysId) + " "
                self.trnsysConn.append(conn)

            temp += "0 "

            temp += str(self.typeNumber)
            temp += " " * (descConnLength - len(temp))
            self.exportConnsString = temp

            return temp + "!" + str(self.trnsysId) + " : " + str(self.displayName) + "\n"

    def exportMassFlows(self):
        if not self.isTempering:
            resStr = "xFrac" + self.displayName + " = " + str(self.positionForMassFlowSolver) + "\n"
            equationNr = 1
            return resStr, equationNr
        else:
            return "", 0

    def exportDivSetting1(self):
        if self.isTempering:
            constants = 1
            f = "T_set_" + self.displayName + "=50\n"
            return f, constants
        else:
            return "", 0

    def exportDivSetting2(self, nUnit):
        if self.isTempering:
            f = ""
            nUnit = nUnit + 1
            f += "UNIT %d TYPE 811 ! Passive Divider for heating \n" % nUnit
            f += "PARAMETERS 1" + "\n"
            f += "5 !Nb.of iterations before fixing the value \n"
            f += "INPUTS 4 \n"

            if (
                self.outputs[0].pos().y() == self.inputs[0].pos().y()
                or self.outputs[0].pos().x() == self.inputs[0].pos().x()
            ):
                first = self.inputs[0]
                second = self.inputs[1]

            f += "T" + first.connectionList[0].displayName + "\n"
            f += "T" + second.connectionList[0].displayName + "\n"
            f += "Mfr" + self.outputs[0].connectionList[0].displayName + "\n"

            f += "T_set_" + self.displayName + "\n"
            f += "*** INITIAL INPUT VALUES" + "\n"
            f += "35.0 21.0 800.0 T_set_" + self.displayName + "\n"

            f += "EQUATIONS 1\n"
            f += "xFrac" + self.displayName + " =  1.-[%d,5] \n\n" % nUnit

            return f, nUnit
        else:
            return "", nUnit

    def exportParametersFlowSolver(self, descConnLength):
        temp = ""

        for o in self.outputs:
            for c in o.connectionList:
                    temp = temp + str(c.trnsysId) + " "
                    self.trnsysConn.append(c)

        for i in self.inputs:
            # Either left or right
            tempArr = []

            for c in i.connectionList:
                # On same hight as output
                if i.pos().x() == self.outputs[0].pos().x() or i.pos().y() == self.outputs[0].y():
                    tempArr.insert(0, c)
                else:
                    tempArr.append(c)

            for conn in tempArr:
                temp = temp + str(conn.trnsysId) + " "
                self.trnsysConn.append(conn)

        temp += str(self.typeNumber)
        temp += " " * (descConnLength - len(temp))
        self.exportConnsString = temp
        f = temp + "!" + str(self.trnsysId) + " : " + str(self.displayName) + "\n"

        return f

    def exportInputsFlowSolver1(self):
        temp1 = "xFrac" + self.displayName
        self.exportInputName = " " + temp1 + " "
        return self.exportInputName, 1

    def exportOutputsFlowSolver(self, prefix, abc, equationNumber, simulationUnit):
        if self.isVisible():
            tot = ""
            for i in range(0, 3):
                temp = (
                    prefix
                    + self.displayName
                    + "_"
                    + abc[i]
                    + "=["
                    + str(simulationUnit)
                    + ","
                    + str(equationNumber)
                    + "]\n"
                )
                tot += temp
                self.exportEquations.append(temp)
                # nEqUsed += 1  # DC
                equationNumber += 1  # DC-ERR
            return tot, equationNumber, 3
        else:
            return "", equationNumber + 1, 0

    def exportPipeAndTeeTypesForTemp(self, startingUnit):
        if self.isVisible():
            f = ""
            unitNumber = startingUnit
            tNr = 929  # Temperature calculation from a tee-piece

            unitText = ""
            ambientT = 20

            equationConstant = 1

            unitText += "UNIT " + str(unitNumber) + " TYPE " + str(tNr) + "\n"
            unitText += "!" + self.displayName + "\n"
            unitText += "PARAMETERS 0\n"
            unitText += "INPUTS 6\n"

            for s in self.exportEquations:
                unitText += s[0 : s.find("=")] + "\n"

            for it in self.trnsysConn:
                unitText += "T" + it.displayName + "\n"

            unitText += "***Initial values\n"
            unitText += 3 * "0 " + 3 * (str(ambientT) + " ") + "\n"

            unitText += "EQUATIONS 1\n"
            unitText += "T" + self.displayName + "= [" + str(unitNumber) + "," + str(equationConstant) + "]\n"

            unitNumber += 1
            f += unitText + "\n"

            return f, unitNumber
        else:
            return "", startingUnit



# pylint: skip-file
# type: ignore

import os

from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import (
    QDialog,
    QLabel,
    QLineEdit,
    QPushButton,
    QCheckBox,
    QHBoxLayout,
    QGridLayout,
    QMessageBox,
    QFileDialog,
    QVBoxLayout,
    QTableWidget,
    QTableWidgetItem,
)
from PyQt5.QtGui import QColor


class FileOrderingDialog(QDialog):
    def __init__(self, fileList, parent=None):
        super(FileOrderingDialog, self).__init__(parent)
        self.fileList = fileList
        self.names = []
        layout = QVBoxLayout()

        self.table = QTableWidget(len(fileList), 2)
        self.table.setMinimumSize(800, 200)
        self.table.setColumnWidth(0, 700)
        self.table.setHorizontalHeaderItem(0, QTableWidgetItem("File"))
        self.table.setHorizontalHeaderItem(1, QTableWidgetItem("Priority"))

        for i in range(0, len(self.fileList)):
            item = QTableWidgetItem(self.fileList[i])
            item.setFlags(Qt.ItemIsSelectable | Qt.ItemIsEnabled)
            self.table.setItem(i, 0, item)

        self.tipText = QLabel("Input 0 to omit from run.config")
        self.okButton = QPushButton("OK")
        self.cancelButton = QPushButton("Cancel")

        buttonLayout = QHBoxLayout()
        buttonLayout.addStretch()
        buttonLayout.addWidget(self.tipText)
        buttonLayout.addWidget(self.okButton)
        buttonLayout.addWidget(self.cancelButton)
        layout.addWidget(self.table)
        layout.addLayout(buttonLayout)
        self.setLayout(layout)

        self.changed_items = []
        # self.table.itemChanged.connect(self.log_change)

        self.okButton.clicked.connect(self.acceptedEdit)
        self.cancelButton.clicked.connect(self.cancel)
        self.setWindowTitle("Set Priority")
        self.show()

    def acceptedEdit(self):
        print("Accepting")
        self.getUserInputs()
        msgb = QMessageBox()

        if not self.checkAllAreIntegers(self.changed_items):
            msgb.setText("Invalid input!")
            msgb.exec_()
            return

        if not self.checkListlength(self.changed_items, self.fileList):
            msgb.setText("Invalid inputs!")
            msgb.exec_()
            return

        self.customSortPaths()
        self.updateConfig()
        self.close()

    def cancel(self):
        self.close()

    def getUserInputs(self):
        self.changed_items.clear()
        for i in range(0, self.table.rowCount()):
            data = self.table.item(i, 1)
            if not data == None and not data == "":
                self.changed_items.append(data.text())

    def customSortPaths(self):
        self.sortedPaths = [x for _, x in sorted(zip(self.changed_items, self.fileList))]
        self.deleteUnwanted()
        print(self.sortedPaths)

    def deleteUnwanted(self):
        zeroCounts = 0
        print("self.changeditems:", self.changed_items)
        for value in self.changed_items:
            if int(value) == 0:
                zeroCounts += 1
        self.finalPathList = self.sortedPaths[zeroCounts:]

    def updateConfig(self):
        config = self.parent().configToEdit

        with open(config, "r") as file:
            lines = file.readlines()
            lines.append("\n")

        header = "LOCAL$ generic\head.ddck\n"
        end = "LOCAL$ generic\end.ddck"

        lines.append(header)
        for items in self.finalPathList:
            item = items.replace("/", "\\")
            item = item.split("ddck\\")[-1]
            print("Item:", item)
            stringToAppend = item
            print(stringToAppend)
            lines.append("LOCAL$ " + stringToAppend + "\n")
        lines.append(end)

        with open(config, "w") as file:
            file.writelines(lines)

    def checkAllAreIntegers(self, list):
        for items in list:
            if not items.isdigit():
                return False
        return True

    def checkListlength(self, list1, list2):
        if len(list1) == len(list2):
            return True
        else:
            return False

    # def log_change(self, item):
    #     self.table.blockSignals(True)
    #     item.setBackground(QColor("red"))
    #     self.table.blockSignals(False)
    #     self.changed_items.append(item)
    #     print(item.text(), item.column(), item.row())
    #
    # def update(self):
    #     print("Updating ")
    #     for item in self.changed_items:
    #         self.table.blockSignals(True)
    #         item.setBackground(QColor("red"))
    #         self.table.blockSignals(False)
    #         self.writeToDatabase(item)
    #
    # def writeToDatabase(self, item):
    #     text, col, row = item.text(), item.column(), item.row()



# pylint: skip-file
# type: ignore

import os
import glob
from PyQt5.QtWidgets import *


class configFile:
    def __init__(self, configPath, editor):
        self.projectFolder = os.path.split(configPath)[0]
        self.editor = editor
        self.configPath = configPath
        if os.path.isfile(self.configPath):
            infile = open(self.configPath, "r")
            self.lines = infile.readlines()
            self.fileExists = True
            self.updateConfig()
        else:
            self.fileExists = False
            qmb = QMessageBox()
            qmb.setText("Please add run.config in %s" % self.projectFolder)
            qmb.setStandardButtons(QMessageBox.Ok)
            qmb.setDefaultButton(QMessageBox.Ok)
            qmb.exec()

    def statementChecker(self, keyword):
        for i in range(len(self.lines)):
            if keyword in self.lines[i]:
                return i
        return -1

    def updateConfig(self):
        """
        Updates the run.config-file with the current project folder, the nameRef, and the projectFolder and adds the
        ddcks of the current project.

        Parameters
        ----------

        Returns
        -------

        """
        originalNumberOfLines = len(self.lines)
        for i in range(originalNumberOfLines):
            try:
                if self.lines[originalNumberOfLines - 1 - i][:9] == "PROJECT$ ":
                    del self.lines[originalNumberOfLines - 1 - i]
            except:
                pass

        ddckPath = os.path.join(self.projectFolder, "ddck")

        linePaths = self.statementChecker("#PATHS#")
        if linePaths == -1:
            self.lines += [
                "\n",
                "##################PATHS##################",
                "\n",
                "\n",
                'string PROJECT$ "%s"\n' % ddckPath,
            ]
        else:
            lineProjectPathDdck = self.statementChecker("PROJECT$ ")
            if lineProjectPathDdck == -1:
                self.lines.insert(linePaths + 2, 'string PROJECT$ "%s"\n' % ddckPath)
            else:
                self.lines[lineProjectPathDdck] = 'string PROJECT$ "%s"\n' % ddckPath

        lineNameRef = self.statementChecker("string nameRef")
        if lineNameRef != -1:
            self.lines[lineNameRef] = 'string nameRef "%s"\n' % os.path.split(self.projectFolder)[-1]

        lineProjectPath = self.statementChecker("string projectPath")
        if lineProjectPath == -1:
            self.lines.insert(lineNameRef, 'string projectPath "%s"\n' % self.projectFolder)
        else:
            self.lines[lineProjectPath] = 'string projectPath "%s"\n' % self.projectFolder

        lineDdck = self.statementChecker("USED DDCKs")
        if lineDdck == -1:
            self.lines += ["\n", "#############USED DDCKs##################", "\n", "\n", "\n"]
            lineDdck = len(self.lines) - 2

        ddckFiles = glob.glob(os.path.join(ddckPath, "**/*.ddck"), recursive=True)
        for i in range(len(ddckFiles)):
            ddckFiles[i] = ddckFiles[i].replace(ddckPath + "\\", "")
            ddckFiles[i] = ddckFiles[i].replace(".ddck", "")
        for i in range(len(ddckFiles)):
            if "head" in ddckFiles[i]:
                ddckFiles.insert(0, ddckFiles.pop(i))
        for i in range(len(ddckFiles)):
            if "end" in ddckFiles[i]:
                ddckFiles.insert(len(ddckFiles) - 1, ddckFiles.pop(i))
        ddcksEntered = 0
        for ddck in ddckFiles:
            self.lines.insert(lineDdck + 2 + ddcksEntered, "PROJECT$ " + ddck + "\n")
            ddcksEntered += 1

        outfile = open(self.configPath, "w")
        outfile.writelines(self.lines)
        outfile.close()

        # def checkConfigDuplicate(self, lines, keyword, replacement):
        #     """
        #     Checks whether a certain keyword occurs in a file split into lines. If the keyword appears in a line, this line
        #     is replaced with the replacement. It returns a Boolean that is true, if the statement was in none of the lines.
        #
        #     Parameters
        #     ----------
        #     lines : list of str
        #     keyword : str
        #     replacement : str
        #
        #     Returns
        #     -------
        #     notInConfigFile : bool
        #
        #     """
        #     notInConfigFile = True
        #     for i in range(len(lines)):
        #         if keyword in lines[i]:
        #             lines[i] = replacement
        #             notInConfigFile = False
        #     return notInConfigFile



# pylint: skip-file
# type: ignore

import os
import shutil
import typing as _tp

from PyQt5.QtCore import QSize, Qt
from PyQt5.QtGui import QPixmap
from PyQt5.QtWidgets import QTreeView

from trnsysGUI.BlockItem import BlockItem
from trnsysGUI.MyQFileSystemModel import MyQFileSystemModel
from trnsysGUI.MyQTreeView import MyQTreeView
from trnsysGUI.PortItem import PortItem
import trnsysGUI.images as _img


class AirSourceHP(BlockItem):
    def __init__(self, trnsysType, parent, **kwargs):
        super(AirSourceHP, self).__init__(trnsysType, parent, **kwargs)

        self.inputs.append(PortItem("i", 2, self))
        self.outputs.append(PortItem("o", 2, self))
        self.path = ""
        self.loadedFiles = []

        self.changeSize()
        self.addTree()

    def _getImageAccessor(self) -> _tp.Optional[_img.ImageAccessor]:
        return _img.AIR_SOURCE_HP_SVG

    def changeSize(self):
        # self.logger.debug("passing through c change size")
        w = self.w
        h = self.h

        """ Resize block function """
        delta = 20
        deltaH = self.h / 10

        # Limit the block size:
        if h < 20:
            h = 20
        if w < 40:
            w = 40
        # center label:
        rect = self.label.boundingRect()
        lw, lh = rect.width(), rect.height()
        lx = (w - lw) / 2
        self.label.setPos(lx, h)

        self.origInputsPos = [[w, delta]]
        self.origOutputsPos = [[w, h - delta]]
        self.inputs[0].setPos(self.origInputsPos[0][0], self.origInputsPos[0][1])
        self.outputs[0].setPos(self.origOutputsPos[0][0], self.origOutputsPos[0][1])

        self.updateFlipStateH(self.flippedH)
        self.updateFlipStateV(self.flippedV)

        self.inputs[0].side = (self.rotationN + 2 - 2 * self.flippedH) % 4
        self.outputs[0].side = (self.rotationN + 2 - 2 * self.flippedH) % 4

        return w, h

    def addTree(self):
        """
        When a blockitem is added to the main window.
        A file explorer for that item is added to the right of the main window by calling this method
        """
        self.logger.debug(self.parent.parent())
        pathName = self.displayName
        if self.parent.parent().projectPath == "":
            # self.path = os.path.dirname(__file__)
            # self.path = os.path.join(self.path, 'default')
            self.path = self.parent.parent().projectFolder
            # now = datetime.now()
            # self.fileName = now.strftime("%Y%m%d%H%M%S")
            # self.path = os.path.join(self.path, self.fileName)
        else:
            self.path = self.parent.parent().projectPath
        self.path = os.path.join(self.path, "ddck")
        self.path = os.path.join(self.path, pathName)
        if not os.path.exists(self.path):
            os.makedirs(self.path)

        self.model = MyQFileSystemModel()
        self.model.setRootPath(self.path)
        self.model.setName(self.displayName)
        self.tree = MyQTreeView(self.model, self)
        self.tree.setModel(self.model)
        self.tree.setRootIndex(self.model.index(self.path))
        self.tree.setObjectName("%sTree" % self.displayName)
        # for i in range(1, self.model.columnCount()-1):
        #     self.tree.hideColumn(i)
        self.tree.setMinimumHeight(200)
        self.tree.setSortingEnabled(True)
        self.parent.parent().splitter.addWidget(self.tree)

    def updateTreePath(self, path):
        """
        When the user chooses the project path for the file explorers, this method is called
        to update the root path.
        """
        pathName = self.displayName
        self.path = os.path.join(path, "ddck")
        self.path = os.path.join(self.path, pathName)
        if not os.path.exists(self.path):
            os.makedirs(self.path)
        self.model.setRootPath(self.path)
        self.tree.setRootIndex(self.model.index(self.path))

    def deleteBlock(self):
        """
        Overridden method to also delete folder
        """
        self.logger.debug("Block " + str(self) + " is deleting itself (" + self.displayName + ")")
        self.deleteConns()
        # self.logger.debug("self.parent.parent" + str(self.parent.parent()))
        self.parent.parent().trnsysObj.remove(self)
        self.logger.debug("deleting block " + str(self) + self.displayName)
        # self.logger.debug("self.scene is" + str(self.parent.scene()))
        self.parent.scene().removeItem(self)
        widgetToRemove = self.parent.parent().findChild(QTreeView, self.displayName + "Tree")
        shutil.rmtree(self.path)
        self.deleteLoadedFile()
        try:
            widgetToRemove.hide()
        except AttributeError:
            self.logger.debug("Widget doesnt exist!")
        else:
            self.logger.debug("Deleted widget")
        del self

    def setName(self, newName):
        """
        Overridden method to also change folder name
        """
        self.displayName = newName
        self.label.setPlainText(newName)
        self.model.setName(self.displayName)
        self.tree.setObjectName("%sTree" % self.displayName)
        self.logger.debug(os.path.dirname(self.path))
        # destPath = str(os.path.dirname(self.path))+'\\AirSourceHP_'+self.displayName
        destPath = os.path.join(os.path.split(self.path)[0], self.displayName)
        if os.path.exists(self.path):
            os.rename(self.path, destPath)
            self.path = destPath
            self.logger.debug(self.path)



# pylint: skip-file
# type: ignore

import glob
import os
import shutil
import typing as _tp

from PyQt5.QtWidgets import QTreeView

from trnsysGUI.BlockItem import BlockItem
from trnsysGUI.MyQFileSystemModel import MyQFileSystemModel
from trnsysGUI.MyQTreeView import MyQTreeView
from trnsysGUI.PortItem import PortItem
import trnsysGUI.images as _img


class HeatPump(BlockItem):
    def __init__(self, trnsysType, parent, **kwargs):
        super().__init__(trnsysType, parent, **kwargs)

        self.inputs.append(PortItem("i", 0, self))
        self.inputs.append(PortItem("i", 2, self))
        self.outputs.append(PortItem("o", 0, self))
        self.outputs.append(PortItem("o", 2, self))
        self.loadedFiles = []

        # For restoring correct order of trnsysObj list
        self.childIds = []
        self.childIds.append(self.trnsysId)
        self.childIds.append(self.parent.parent().idGen.getTrnsysID())

        self.changeSize()
        self.addTree()

    def _getImageAccessor(self) -> _tp.Optional[_img.ImageAccessor]:
        return _img.HP_SVG

    def changeSize(self):
        w = self.w
        h = self.h

        """ Resize block function """
        delta = 20

        # Limit the block size:
        if h < 20:
            h = 20
        if w < 40:
            w = 40
        # center label:
        rect = self.label.boundingRect()
        lw, lh = rect.width(), rect.height()
        lx = (w - lw) / 2
        self.label.setPos(lx, h)

        self.origInputsPos = [[0, delta], [w, delta]]
        self.origOutputsPos = [[0, h - delta], [w, h - delta]]
        self.inputs[0].setPos(self.origInputsPos[0][0], self.origInputsPos[0][1])
        self.inputs[1].setPos(self.origInputsPos[1][0], self.origInputsPos[1][1])
        self.outputs[0].setPos(self.origOutputsPos[0][0], self.origOutputsPos[0][1])
        self.outputs[1].setPos(self.origOutputsPos[1][0], self.origOutputsPos[1][1])

        self.updateFlipStateH(self.flippedH)
        self.updateFlipStateV(self.flippedV)

        self.inputs[0].side = (self.rotationN + 2 * self.flippedH) % 4
        self.inputs[1].side = (self.rotationN + 2 - 2 * self.flippedH) % 4

        self.outputs[0].side = (self.rotationN + 2 * self.flippedH) % 4
        self.outputs[1].side = (self.rotationN + 2 - 2 * self.flippedH) % 4
        return w, h

    def encode(self):
        if self.isVisible():
            self.logger.debug("Encoding a HeatPump")

            # childIdList = []

            portListInputs = []
            portListOutputs = []

            for p in self.inputs:
                portListInputs.append(p.id)
            for p in self.outputs:
                portListOutputs.append(p.id)

            dct = {}
            dct[".__BlockDict__"] = True
            dct["BlockName"] = self.name
            dct["BlockDisplayName"] = self.displayName
            dct["PortsIDIn"] = portListInputs
            dct["PortsIDOut"] = portListOutputs
            dct["HeatPumpPosition"] = (float(self.pos().x()), float(self.pos().y()))
            dct["ID"] = self.id
            dct["trnsysID"] = self.trnsysId
            dct["childIds"] = self.childIds
            dct["FlippedH"] = self.flippedH
            dct["FlippedV"] = self.flippedH
            dct["RotationN"] = self.rotationN
            dct["GroupName"] = self.groupName

            dictName = "Block-"

            return dictName, dct

    def decode(self, i, resBlockList):
        self.logger.debug("Loading a HeatPump block")

        self.flippedH = i["FlippedH"]
        self.flippedV = i["FlippedV"]
        self.childIds = i["childIds"]
        self.displayName = i["BlockDisplayName"]
        self.changeSize()

        for x in range(len(self.inputs)):
            self.inputs[x].id = i["PortsIDIn"][x]
            self.logger.debug("Input at heatExchanger")

        for x in range(len(self.outputs)):
            self.outputs[x].id = i["PortsIDOut"][x]
            self.logger.debug("Output at heatExchanger")

        self.setPos(float(i["HeatPumpPosition"][0]), float(i["HeatPumpPosition"][1]))
        self.trnsysId = i["trnsysID"]
        self.id = i["ID"]

        self.groupName = "defaultGroup"
        self.setBlockToGroup(i["GroupName"])

        resBlockList.append(self)

    def decodePaste(self, i, offset_x, offset_y, resConnList, resBlockList, **kwargs):
        self.logger.debug("Loading a HeatPump in Decoder")

        self.changeSize()

        for x in range(len(self.inputs)):
            self.inputs[x].id = i["PortsIDIn"][x]
            self.logger.debug("Input at heatExchanger")

        for x in range(len(self.outputs)):
            self.outputs[x].id = i["PortsIDOut"][x]
            self.logger.debug("Output at heatExchanger")

        self.setPos(float(i["HeatPumpPosition"][0]) + offset_x, float(i["HeatPumpPosition"][1] + offset_y))
        self.groupName = "defaultGroup"
        self.setBlockToGroup(i["GroupName"])

        resBlockList.append(self)

    def exportBlackBox(self):
        equation = []
        files = glob.glob(os.path.join(self.path, "**/*.ddck"), recursive=True)
        if not (files):
            status = "noDdckFile"
            for i in range(1, 3):
                equation.append("T" + self.displayName + "X" + str(i) + "=1")
        else:
            status = "noDdckEntry"
        lines = []
        for file in files:
            infile = open(file, "r")
            lines += infile.readlines()
        for i in range(len(lines)):
            if "output" in lines[i].lower() and "to" in lines[i].lower() and "hydraulic" in lines[i].lower():
                counter = 1
                for j in range(i, len(lines) - i):
                    if lines[j][0] == "T":
                        outputT = lines[j].split("=")[0].replace(" ", "")
                        equation.append("T" + self.displayName + "X" + str(counter) + "=1 ! suggestion: " + outputT)
                        counter += 1
                    if counter == 3:
                        status = "success"
                        break
                break

        return status, equation

    def exportParametersFlowSolver(self, descConnLength):
        f = ""
        for i in range(len(self.inputs)):
            for c in self.inputs[i].connectionList:
                if len(self.outputs[i].connectionList) > 0:
                    if i == 0:
                        temp = (
                            str(c.trnsysId) + " " + str(self.outputs[i].connectionList[0].trnsysId) + " 0 0 "
                        )  # + str(t.childIds[0])
                        temp += " " * (descConnLength - len(temp))

                        # HeatPump will have a two-liner exportConnString
                        self.exportConnsString += temp + "\n"
                        f += temp + "!" + str(self.childIds[0]) + " : " + self.displayName + "HeatPump" + "\n"

                    elif i == 1:
                        temp = (
                            str(c.trnsysId) + " " + str(self.outputs[i].connectionList[0].trnsysId) + " 0 0 "
                        )  # + str(t.childIds[1])
                        temp += " " * (descConnLength - len(temp))

                        # HeatPump will have a two liner exportConnString
                        self.exportConnsString += temp + "\n"
                        f += temp + "!" + str(self.childIds[1]) + " : " + self.displayName + "Evap" + "\n"
                    else:
                        f += "Error: There are more inputs than trnsysIds" + "\n"

                    # Presumably used only for storing the order of connections
                    self.trnsysConn.append(c)
                    self.trnsysConn.append(self.outputs[i].connectionList[0])

                else:
                    f += "Output of HeatPump for input[{0}] is not connected ".format(i) + "\n"

        return f

    def exportInputsFlowSolver1(self):
        return "0,0 0,0 ", 2

    def exportInputsFlowSolver2(self):
        f = ""
        f += " " + str(self.exportInitialInput) + " " + str(self.exportInitialInput) + " "
        return f, 2

    def exportOutputsFlowSolver(self, prefix, abc, equationNumber, simulationUnit):
        tot = ""
        for j in range(2):
            for i in range(0, 3):

                if i < 2:
                    temp = (
                        prefix
                        + self.displayName
                        + "-Hp-Side"
                        + str(j)
                        + "_"
                        + abc[i]
                        + "=["
                        + str(simulationUnit)
                        + ","
                        + str(equationNumber)
                        + "]\n"
                    )
                    tot += temp
                    self.exportEquations.append(temp)
                    # nEqUsed += 1  # DC
                equationNumber += 1  # DC-ERROR it should count anyway

        return tot, equationNumber, 4

    def getSubBlockOffset(self, c):
        for i in range(2):
            if (
                self.inputs[i] == c.toPort
                or self.inputs[i] == c.fromPort
                or self.outputs[i] == c.toPort
                or self.outputs[i] == c.fromPort
            ):
                return i

    def addTree(self):
        """
        When a blockitem is added to the main window.
        A file explorer for that item is added to the right of the main window by calling this method
        """
        self.logger.debug(self.parent.parent())
        pathName = self.displayName
        if self.parent.parent().projectPath == "":
            # self.path = os.path.dirname(__file__)
            # self.path = os.path.join(self.path, 'default')
            self.path = self.parent.parent().projectFolder
            # now = datetime.now()
            # self.fileName = now.strftime("%Y%m%d%H%M%S")
            # self.path = os.path.join(self.path, self.fileName)
        else:
            self.path = self.parent.parent().projectPath
        self.path = os.path.join(self.path, "ddck")
        self.path = os.path.join(self.path, pathName)
        if not os.path.exists(self.path):
            os.makedirs(self.path)

        self.model = MyQFileSystemModel()
        self.model.setRootPath(self.path)
        self.model.setName(self.displayName)
        self.tree = MyQTreeView(self.model, self)
        self.tree.setModel(self.model)
        self.tree.setRootIndex(self.model.index(self.path))
        self.tree.setObjectName("%sTree" % self.displayName)
        for i in range(1, self.model.columnCount() - 1):
            self.tree.hideColumn(i)
        self.tree.setMinimumHeight(200)
        self.tree.setSortingEnabled(True)
        self.parent.parent().splitter.addWidget(self.tree)

    # def loadFile(self, file):
    #     filePath = self.parent.parent().projectPath
    #     msgB = QMessageBox()
    #     if filePath == '':
    #         msgB.setText("Please select a project path before loading!")
    #         msgB.exec_()
    #     else:
    #         self.logger.debug("file loaded into %s" % filePath)
    #         shutil.copy(file, filePath)

    def updateTreePath(self, path):
        """
        When the user chooses the project path for the file explorers, this method is called
        to update the root path.
        """
        pathName = self.displayName
        self.path = os.path.join(path, "ddck")
        self.path = os.path.join(self.path, pathName)
        if not os.path.exists(self.path):
            os.makedirs(self.path)
        self.model.setRootPath(self.path)
        self.tree.setRootIndex(self.model.index(self.path))

    def deleteBlock(self):
        """
        Overridden method to also delete folder
        """
        self.logger.debug("Block " + str(self) + " is deleting itself (" + self.displayName + ")")
        self.deleteConns()
        # self.logger.debug("self.parent.parent" + str(self.parent.parent()))
        self.parent.parent().trnsysObj.remove(self)
        self.logger.debug("deleting block " + str(self) + self.displayName)
        # self.logger.debug("self.scene is" + str(self.parent.scene()))
        self.parent.scene().removeItem(self)
        widgetToRemove = self.parent.parent().findChild(QTreeView, self.displayName + "Tree")
        shutil.rmtree(self.path)
        self.deleteLoadedFile()
        try:
            widgetToRemove.hide()
        except AttributeError:
            self.logger.debug("Widget doesnt exist!")
        else:
            self.logger.debug("Deleted widget")
        del self

    def setName(self, newName):
        """
        Overridden method to also change folder name
        """
        self.displayName = newName
        self.label.setPlainText(newName)
        self.model.setName(self.displayName)
        self.tree.setObjectName("%sTree" % self.displayName)
        self.logger.debug(os.path.dirname(self.path))
        # destPath = str(os.path.dirname(self.path))+'\\HP_'+self.displayName
        destPath = os.path.join(os.path.split(self.path)[0], self.displayName)
        if os.path.exists(self.path):
            os.rename(self.path, destPath)
            self.path = destPath
            self.logger.debug(self.path)



# pylint: skip-file
# type: ignore

import os
import shutil
import typing as _tp

from PyQt5.QtCore import QSize
from PyQt5.QtGui import QPixmap
from PyQt5.QtWidgets import QTreeView

from trnsysGUI.BlockItem import BlockItem
from trnsysGUI.MyQFileSystemModel import MyQFileSystemModel
from trnsysGUI.MyQTreeView import MyQTreeView
from trnsysGUI.PortItem import PortItem
import trnsysGUI.images as _img


class HeatPumpTwoHx(BlockItem):
    def __init__(self, trnsysType, parent, **kwargs):
        super(HeatPumpTwoHx, self).__init__(trnsysType, parent, **kwargs)

        self.inputs.append(PortItem("i", 0, self))
        self.inputs.append(PortItem("i", 2, self))
        self.inputs.append(PortItem("i", 2, self))

        self.outputs.append(PortItem("o", 0, self))
        self.outputs.append(PortItem("o", 2, self))
        self.outputs.append(PortItem("o", 2, self))
        self.loadedFiles = []

        # For restoring correct order of trnsysObj list
        self.childIds = []
        self.childIds.append(self.trnsysId)
        self.childIds.append(self.parent.parent().idGen.getTrnsysID())
        self.childIds.append(self.parent.parent().idGen.getTrnsysID())

        self.changeSize()
        self.addTree()

    def _getImageAccessor(self) -> _tp.Optional[_img.ImageAccessor]:
        return _img.HP_TWO_HX_SVG

    def changeSize(self):
        w = self.w
        h = self.h

        """ Resize block function """
        delta = 20

        # Limit the block size:
        if h < 20:
            h = 20
        if w < 40:
            w = 40
        # center label:
        rect = self.label.boundingRect()
        lw, lh = rect.width(), rect.height()
        lx = (w - lw) / 2
        self.label.setPos(lx, h)

        self.origInputsPos = [[0, delta], [w, delta], [w, h - 2 * delta]]
        self.origOutputsPos = [[0, h - delta], [w, 2 * delta], [w, h - delta]]
        self.inputs[0].setPos(self.origInputsPos[0][0], self.origInputsPos[0][1])
        self.inputs[1].setPos(self.origInputsPos[1][0], self.origInputsPos[1][1])
        self.inputs[2].setPos(self.origInputsPos[2][0], self.origInputsPos[2][1])
        self.outputs[0].setPos(self.origOutputsPos[0][0], self.origOutputsPos[0][1])
        self.outputs[1].setPos(self.origOutputsPos[1][0], self.origOutputsPos[1][1])
        self.outputs[2].setPos(self.origOutputsPos[2][0], self.origOutputsPos[2][1])

        self.updateFlipStateH(self.flippedH)
        self.updateFlipStateV(self.flippedV)

        self.inputs[0].side = (self.rotationN + 2 * self.flippedH) % 4
        self.inputs[1].side = (self.rotationN + 2 - 2 * self.flippedH) % 4
        self.inputs[2].side = (self.rotationN + 2 - 2 * self.flippedH) % 4

        self.outputs[0].side = (self.rotationN + 2 * self.flippedH) % 4
        self.outputs[1].side = (self.rotationN + 2 - 2 * self.flippedH) % 4
        self.outputs[2].side = (self.rotationN + 2 - 2 * self.flippedH) % 4
        return w, h

    def encode(self):
        if self.isVisible():
            self.logger.debug("Encoding a HeatPump")

            portListInputs = []
            portListOutputs = []

            for p in self.inputs:
                portListInputs.append(p.id)
            for p in self.outputs:
                portListOutputs.append(p.id)

            dct = {}
            dct[".__BlockDict__"] = True
            dct["BlockName"] = self.name
            dct["BlockDisplayName"] = self.displayName
            dct["PortsIDIn"] = portListInputs
            dct["PortsIDOut"] = portListOutputs
            dct["HeatPumpPosition"] = (float(self.pos().x()), float(self.pos().y()))
            dct["ID"] = self.id
            dct["trnsysID"] = self.trnsysId
            dct["childIds"] = self.childIds
            dct["FlippedH"] = self.flippedH
            dct["FlippedV"] = self.flippedH
            dct["RotationN"] = self.rotationN
            dct["GroupName"] = self.groupName

            dictName = "Block-"

            return dictName, dct

    def decode(self, i, resBlockList):
        self.flippedH = i["FlippedH"]
        self.flippedV = i["FlippedV"]
        self.childIds = i["childIds"]
        self.displayName = i["BlockDisplayName"]
        self.changeSize()

        for x in range(len(self.inputs)):
            self.inputs[x].id = i["PortsIDIn"][x]
            self.logger.debug("Input at heatExchanger")

        for x in range(len(self.outputs)):
            self.outputs[x].id = i["PortsIDOut"][x]
            self.logger.debug("Output at heatExchanger")

        self.setPos(float(i["HeatPumpPosition"][0]), float(i["HeatPumpPosition"][1]))
        self.trnsysId = i["trnsysID"]
        self.id = i["ID"]

        self.groupName = "defaultGroup"
        self.setBlockToGroup(i["GroupName"])

        resBlockList.append(self)

    def decodePaste(self, i, offset_x, offset_y, resConnList, resBlockList, **kwargs):
        self.flippedH = i["FlippedH"]
        self.flippedV = i["FlippedV"]

        self.changeSize()

        for x in range(len(self.inputs)):
            self.inputs[x].id = i["PortsIDIn"][x]
            self.logger.debug("Input at heatExchanger")

        for x in range(len(self.outputs)):
            self.outputs[x].id = i["PortsIDOut"][x]
            self.logger.debug("Output at heatExchanger")

        self.setPos(float(i["HeatPumpPosition"][0] + offset_x), float(i["HeatPumpPosition"][1] + offset_y))
        # self.trnsysId = i["trnsysID"]
        # self.id = i["ID"]

        self.groupName = "defaultGroup"
        self.setBlockToGroup(i["GroupName"])
        resBlockList.append(self)

    def exportBlackBox(self):
        equations = ["T" + self.displayName + "X1" + "=1"]
        equations.append("T" + self.displayName + "X2" + "=1")
        equations.append("T" + self.displayName + "X3" + "=1")
        status = "success"
        return status, equations

    def exportParametersFlowSolver(self, descConnLength):
        f = ""
        for i in range(len(self.inputs)):
            for c in self.inputs[i].connectionList:
                if len(self.outputs[i].connectionList) > 0:
                    # HeatPumpTwoHx exportConnsString has 3 lines
                    if i == 0:
                        temp = (
                            str(c.trnsysId) + " " + str(self.outputs[i].connectionList[0].trnsysId) + " 0 0 "
                        )  # + str(t.childIds[0])
                        temp += " " * (descConnLength - len(temp))

                        self.exportConnsString += temp + "\n"
                        f += temp + "!" + str(self.childIds[0]) + " : " + self.displayName + "Side1" + "\n"

                    elif i == 1:
                        temp = (
                            str(c.trnsysId) + " " + str(self.outputs[i].connectionList[0].trnsysId) + " 0 0 "
                        )  # + str(t.childIds[1])
                        temp += " " * (descConnLength - len(temp))

                        self.exportConnsString += temp + "\n"
                        f += temp + "!" + str(self.childIds[1]) + " : " + self.displayName + "Side2" + "\n"

                    elif i == 2:
                        temp = (
                            str(c.trnsysId) + " " + str(self.outputs[i].connectionList[0].trnsysId) + " 0 0 "
                        )  # + str(t.childIds[1])
                        temp += " " * (descConnLength - len(temp))

                        self.exportConnsString += temp + "\n"
                        f += temp + "!" + str(self.childIds[2]) + " : " + self.displayName + "Side3" + "\n"
                    else:
                        f += "Error: There are more inputs than trnsysIds" + "\n"

                    # Presumably used only for storing the order of connections
                    self.trnsysConn.append(c)
                    self.trnsysConn.append(self.outputs[i].connectionList[0])

                else:
                    f += "Output of ExternalHx for input[{0}] is not connected ".format(i) + "\n"

        return f

    def exportInputsFlowSolver1(self):
        return "0,0 0,0 0,0", 3

    def exportInputsFlowSolver2(self):
        f = ""
        f += (
            " "
            + str(self.exportInitialInput)
            + " "
            + str(self.exportInitialInput)
            + " "
            + str(self.exportInitialInput)
            + " "
        )
        return f, 3

    def exportOutputsFlowSolver(self, prefix, abc, equationNumber, simulationUnit):
        tot = ""
        for j in range(3):
            for i in range(0, 3):
                if i < 2:
                    temp = (
                        prefix
                        + self.displayName
                        + "-Side"
                        + str(j)
                        + "_"
                        + abc[i]
                        + "=["
                        + str(simulationUnit)
                        + ","
                        + str(equationNumber)
                        + "]\n"
                    )
                    tot += temp
                    self.exportEquations.append(temp)
                    # nEqUsed += 1  # DC
                equationNumber += 1  # DC-ERROR it should count anyway

        return tot, equationNumber, 6

    def getSubBlockOffset(self, c):
        for i in range(3):
            if (
                self.inputs[i] == c.toPort
                or self.inputs[i] == c.fromPort
                or self.outputs[i] == c.toPort
                or self.outputs[i] == c.fromPort
            ):
                return i

    def addTree(self):
        """
        When a blockitem is added to the main window.
        A file explorer for that item is added to the right of the main window by calling this method
        """
        self.logger.debug(self.parent.parent())
        pathName = self.displayName
        if self.parent.parent().projectPath == "":
            # self.path = os.path.dirname(__file__)
            # self.path = os.path.join(self.path, 'default')
            self.path = self.parent.parent().projectFolder
            # now = datetime.now()
            # self.fileName = now.strftime("%Y%m%d%H%M%S")
            # self.path = os.path.join(self.path, self.fileName)
        else:
            self.path = self.parent.parent().projectPath
        self.path = os.path.join(self.path, "ddck")
        self.path = os.path.join(self.path, pathName)
        if not os.path.exists(self.path):
            os.makedirs(self.path)

        self.model = MyQFileSystemModel()
        self.model.setRootPath(self.path)
        self.model.setName(self.displayName)
        self.tree = MyQTreeView(self.model, self)
        self.tree.setModel(self.model)
        self.tree.setRootIndex(self.model.index(self.path))
        self.tree.setObjectName("%sTree" % self.displayName)
        for i in range(1, self.model.columnCount() - 1):
            self.tree.hideColumn(i)
        self.tree.setMinimumHeight(200)
        self.tree.setSortingEnabled(True)
        self.parent.parent().splitter.addWidget(self.tree)

    # def loadFile(self, file):
    #     filePath = self.parent.parent().projectPath
    #     msgB = QMessageBox()
    #     if filePath == '':
    #         msgB.setText("Please select a project path before loading!")
    #         msgB.exec_()
    #     else:
    #         self.logger.debug("file loaded into %s" % filePath)
    #         shutil.copy(file, filePath)

    def updateTreePath(self, path):
        """
        When the user chooses the project path for the file explorers, this method is called
        to update the root path.
        """
        pathName = self.displayName
        self.path = os.path.join(path, "ddck")
        self.path = os.path.join(self.path, pathName)
        if not os.path.exists(self.path):
            os.makedirs(self.path)
        self.model.setRootPath(self.path)
        self.tree.setRootIndex(self.model.index(self.path))

    def deleteBlock(self):
        """
        Overridden method to also delete folder
        """
        self.logger.debug("Block " + str(self) + " is deleting itself (" + self.displayName + ")")
        self.deleteConns()
        # self.logger.debug("self.parent.parent" + str(self.parent.parent()))
        self.parent.parent().trnsysObj.remove(self)
        self.logger.debug("deleting block " + str(self) + self.displayName)
        # self.logger.debug("self.scene is" + str(self.parent.scene()))
        self.parent.scene().removeItem(self)
        widgetToRemove = self.parent.parent().findChild(QTreeView, self.displayName + "Tree")
        shutil.rmtree(self.path)
        self.deleteLoadedFile()
        try:
            widgetToRemove.hide()
        except AttributeError:
            self.logger.debug("Widget doesnt exist!")
        else:
            self.logger.debug("Deleted widget")
        del self

    def setName(self, newName):
        """
        Overridden method to also change folder name
        """
        self.displayName = newName
        self.label.setPlainText(newName)
        self.model.setName(self.displayName)
        self.tree.setObjectName("%sTree" % self.displayName)
        self.logger.debug(os.path.dirname(self.path))
        # destPath = str(os.path.dirname(self.path))+'\\HPTwoHx_'+self.displayName
        destPath = os.path.join(os.path.split(self.path)[0], self.displayName)
        if os.path.exists(self.path):
            os.rename(self.path, destPath)
            self.path = destPath
            self.logger.debug(self.path)



# pylint: skip-file
# type: ignore

from PyQt5.QtWidgets import QUndoCommand

import trnsysGUI.Collector
import trnsysGUI.Connector
import trnsysGUI.GenericBlock
import trnsysGUI.HeatPump
import trnsysGUI.HeatPumpTwoHx
import trnsysGUI.IceStorage
import trnsysGUI.Pump
import trnsysGUI.Radiator
import trnsysGUI.StorageTank
import trnsysGUI.TVentil
import trnsysGUI.TeePiece
import trnsysGUI.WTap
import trnsysGUI.WTap_main
import trnsysGUI.Boiler
import trnsysGUI.BlockItem
import trnsysGUI.AirSourceHP
import trnsysGUI.PV


class DeleteBlockCommand(QUndoCommand):
    def __init__(self, bl, descr):
        super(DeleteBlockCommand, self).__init__(descr)
        self.bl = bl
        self.blW = bl.w
        self.blH = bl.h
        self.blParent = bl.parent
        self.blId = bl.id
        self.blTrnsysId = bl.trnsysId

        self.portsIdIn = []
        self.portsIdOut = []
        self.savePortsId()

        self.blFlippedH = bl.flippedH
        self.blFlippedV = bl.flippedV
        self.blRotation = bl.rotationN
        self.blGroupName = bl.groupName
        self.blName = bl.name
        self.bldName = bl.displayName
        self.blPos = bl.pos()

    def redo(self):
        self.bl.deleteBlock()
        self.bl = None

    def undo(self):
        name = self.blName
        if name == "StorageTank":
            bl = trnsysGUI.StorageTank.StorageTank(name, self.blParent)
        elif name == "TeePiece":
            bl = trnsysGUI.TeePiece.TeePiece(name, self.blParent)
        elif name == "TVentil":
            bl = trnsysGUI.TVentil.TVentil(name, self.blParent)
        elif name == "Pump":
            bl = trnsysGUI.Pump.Pump(name, self.blParent)
        elif name == "Collector":
            bl = trnsysGUI.Collector.Collector(name, self.blParent)
        elif name == "HP":
            bl = trnsysGUI.HeatPump.HeatPump(name, self.blParent)
        elif name == "IceStorage":
            bl = trnsysGUI.IceStorage.IceStorage(name, self.blParent)
        elif name == "Radiator":
            bl = trnsysGUI.Radiator.Radiator(name, self.blParent)
        elif name == "WTap":
            bl = trnsysGUI.WTap.WTap(name, self.blParent)
        elif name == "WTap_main":
            bl = trnsysGUI.WTap_mainW.Tap_main(name, self.blParent)
        elif name == "Connector":
            bl = trnsysGUI.Connector.Connector(name, self.blParent)
        elif name == "GenericBlock":
            bl = trnsysGUI.GenericBlock.GenericBlock(name, self.blParent)
        elif name == "HPTwoHx":
            bl = trnsysGUI.HeatPumpTwoHx.HeatPumpTwoHx(name, self.blParent)
        elif name == "Boiler":
            bl = trnsysGUI.Boiler.Boiler(name, self.blParent)
        elif name == "AirSourceHP":
            bl = trnsysGUI.AirSourceHP.AirSourceHP(name, self.blParent)
        elif name == "PV":
            bl = trnsysGUI.PV.PV
        elif name == "GroundSourceHx":
            bl = trnsysGUI.GroundSourceHx.GroundSourceHx(name, self.blParent)
        else:
            bl = trnsysGUI.BlockItem.BlockItem(name, self.blParent)

        bl.trnsysId = self.blTrnsysId
        bl.id = self.blId
        bl.updateFlipStateH(self.blFlippedH)
        bl.updateFlipStateV(self.blFlippedV)
        bl.rotateBlockToN(self.blRotation)
        bl.displayName = self.bldName
        bl.label.setPlainText(bl.displayName)

        bl.groupName = "defaultGroup"
        bl.setBlockToGroup(self.blGroupName)

        bl.setPos(self.blPos)
        self.blParent.scene().addItem(bl)

        bl.oldPos = bl.scenePos()

    def savePortsId(self):
        for p in self.bl.inputs:
            self.portsIdIn.append(p.id)
        for p in self.bl.outputs:
            self.portsIdOut.append(p.id)


# From stackoverflow, this should have helped:

# from trnsysGUI.Collector import Collector
# from trnsysGUI.Connector import Connector
# from trnsysGUI.GenericBlock import GenericBlock
# from trnsysGUI.HeatPump import HeatPump
# from trnsysGUI.HeatPumpTwoHx import HeatPumpTwoHx
# from trnsysGUI.IceStorage import IceStorage
# from trnsysGUI.Pump import Pump
# from trnsysGUI.Radiator import Radiator
# from trnsysGUI.StorageTank import StorageTank
# from trnsysGUI.TVentil import TVentil
# from trnsysGUI.TeePiece import TeePiece
# from trnsysGUI.WTap import WTap
# from trnsysGUI.WTap_main import WTap_main
#
# from trnsysGUI.BlockItem import BlockItem



# pylint: skip-file
# type: ignore

import os
import shutil

from PyQt5.QtCore import QSize
from PyQt5.QtGui import QPixmap
from PyQt5.QtWidgets import QTreeView

from trnsysGUI.BlockItem import BlockItem


class MasterControl(BlockItem):
    def __init__(self, trnsysType, parent, **kwargs):
        super(MasterControl, self).__init__(trnsysType, parent, **kwargs)
        factor = 0.667  # 0.63 for png
        self.w = 100
        self.h = 100
        self.loadedFiles = []

        self.parent.parent().controlExists += 1
        self.createControlDir()

    def createControlDir(self):
        if self.parent.parent().projectPath == "":
            projectPath = self.parent.parent().projectFolder
        else:
            projectPath = self.parent.parent().projectPath

        self.parent.parent().controlDirectory = os.path.join(projectPath, "ddck")
        self.parent.parent().controlDirectory = os.path.join(self.parent.parent().controlDirectory, "Master_Control")
        if not os.path.exists(self.parent.parent().controlDirectory):
            os.makedirs(self.parent.parent().controlDirectory)

    def deleteBlock(self):
        """
        Overridden method to also delete folder
        """
        print("Block " + str(self) + " is deleting itself (" + self.displayName + ")")
        self.deleteConns()
        # print("self.parent.parent" + str(self.parent.parent()))
        self.parent.parent().trnsysObj.remove(self)
        print("deleting block " + str(self) + self.displayName)
        # print("self.scene is" + str(self.parent.scene()))
        self.parent.scene().removeItem(self)
        widgetToRemove = self.parent.parent().findChild(QTreeView, self.displayName + "Tree")
        self.parent.parent().controlExists -= 1
        if self.parent.parent().controlExists == 0:
            shutil.rmtree(self.parent.parent().controlDirectory)

        # self.deleteLoadedFile()
        try:
            widgetToRemove.hide()
        except AttributeError:
            print("Widget doesnt exist!")
        else:
            print("Deleted widget")
        del self



# pylint: skip-file
# type: ignore

from PyQt5.QtCore import QMimeData, Qt
from PyQt5.QtGui import QStandardItemModel


class LibraryModel(QStandardItemModel):
    def __init__(self, parent=None):
        QStandardItemModel.__init__(self, parent)

    def mimeTypes(self):
        return ["component/name"]

    def mimeData(self, idxs):
        mimedata = QMimeData()
        for idx in idxs:
            if idx.isValid():
                txt = self.data(idx, Qt.DisplayRole)
                txt2 = bytearray()
                txt2.extend(map(ord, txt))
                mimedata.setData("component/name", txt2)
        return mimedata



# pylint: skip-file
# type: ignore

import os

from PyQt5.QtWidgets import (
    QDialog,
    QLabel,
    QLineEdit,
    QPushButton,
    QCheckBox,
    QHBoxLayout,
    QGridLayout,
    QMessageBox,
    QFileDialog,
)
from PyQt5.QtGui import QIcon

import trnsysGUI.BlockItem as _bi


class BlockDlg(QDialog):
    def __init__(self, block: _bi.BlockItem, parent=None):
        super(BlockDlg, self).__init__(parent)
        nameLabel = QLabel("Name:")
        self.logger = parent.logger
        self.block = block
        self.le = QLineEdit(self.block.label.toPlainText())
        self.setWindowIcon(QIcon(block.pixmap()))
        self.loadButton = QPushButton("Load")
        self.okButton = QPushButton("OK")
        self.cancelButton = QPushButton("Cancel")

        self.hFlipBoxLabel = QLabel("Horizontal Flip")
        self.vFlipBoxLabel = QLabel("Vertical Flip")

        self.hFlipBox = QCheckBox()
        self.vFlipBox = QCheckBox()

        self.vFlipBox.setTristate(False)
        self.hFlipBox.setTristate(False)
        self.hFlipBox.setCheckState(self.block.flippedH * 2)
        self.vFlipBox.setCheckState(self.block.flippedV * 2)

        flipLayout = QHBoxLayout()
        flipLayout.addWidget(self.hFlipBoxLabel)
        flipLayout.addWidget(self.hFlipBox)
        flipLayout.addWidget(self.vFlipBoxLabel)
        flipLayout.addWidget(self.vFlipBox)

        buttonLayout = QHBoxLayout()
        buttonLayout.addStretch()
        buttonLayout.addWidget(self.loadButton)
        buttonLayout.addWidget(self.okButton)
        buttonLayout.addWidget(self.cancelButton)
        layout = QGridLayout()
        layout.addWidget(nameLabel, 0, 0)
        layout.addWidget(self.le, 0, 1)
        layout.addLayout(flipLayout, 1, 0, 2, 0)
        layout.addLayout(buttonLayout, 2, 0, 2, 0)
        self.setLayout(layout)

        self.setFixedSize(300, 150)

        self.loadButton.clicked.connect(self.loadFile)
        self.okButton.clicked.connect(self.acceptedEdit)
        self.cancelButton.clicked.connect(self.cancel)
        self.hFlipBox.stateChanged.connect(self.setNewFlipStateH)
        self.vFlipBox.stateChanged.connect(self.setNewFlipStateV)
        self.setWindowTitle("Properties")
        self.loadButton.setDisabled(True)
        self.disableLoad()
        self.show()

    def acceptedEdit(self):
        self.logger.debug("Changing displayName")
        newName = self.le.text()
        if newName.lower() == str(self.block.displayName).lower():
            self.close()
        elif newName != "" and not self.nameExists(newName):
            self.block.setName(newName)
            # self.block.label.setPlainText(newName)
            # self.block.displayName = newName
            self.close()
        elif newName == "":
            msgb = QMessageBox()
            msgb.setText("Please Enter a name!")
            msgb.exec()
        elif self.nameExists(newName):
            msgb = QMessageBox()
            msgb.setText("Name already exist!")
            msgb.exec()

    def setNewFlipStateH(self, state):
        self.block.updateFlipStateH(state)

    def setNewFlipStateV(self, state):
        self.block.updateFlipStateV(state)

    def cancel(self):
        self.close()

    def nameExists(self, n):
        for t in self.parent().trnsysObj:
            if str(t.displayName).lower() == n.lower():
                return True
        return False

    # unused
    def loadFile(self):
        self.logger.debug("Opening diagram")
        # self.centralWidget.delBlocks()
        fileName = QFileDialog.getOpenFileName(self, "Open diagram", filter="*.ddck")[0]
        if fileName != "":
            if len(self.block.propertyFile) < 2:
                self.block.propertyFile.append(fileName)
            else:
                self.block.propertyFile.clear()
                self.block.propertyFile.append(fileName)
        else:
            self.logger.debug("No filename chosen")
        pass

    def disableLoad(self):
        if self.block.name == "TeePiece" or self.block.name == "WTap_main":
            self.loadButton.setDisabled(True)



# pylint: skip-file
# type: ignore

import trnsysGUI.imageAccessor as _ia


ImageAccessor = _ia.ImageAccessor


AIR_SOURCE_HP_PNG = _ia.ImageAccessor.createForPackageResource("images/AirSourceHP.png")
AIR_SOURCE_HP_SVG = _ia.ImageAccessor.createForPackageResource("images/AirSourceHP.svg")
BOILER_PNG = _ia.ImageAccessor.createForPackageResource("images/Boiler.png")
BOILER_SVG = _ia.ImageAccessor.createForPackageResource("images/Boiler.svg")
BVI_PNG = _ia.ImageAccessor.createForPackageResource("images/Bvi.png")
COLLECTOR_PNG = _ia.ImageAccessor.createForPackageResource("images/Collector.png")
COLLECTOR_SVG = _ia.ImageAccessor.createForPackageResource("images/Collector.svg")
CONNECTOR_PNG = _ia.ImageAccessor.createForPackageResource("images/Connector.png")
CONNECTOR_SVG = _ia.ImageAccessor.createForPackageResource("images/Connector.svg")
ELECTRIC_ROD_SVG = _ia.ImageAccessor.createForPackageResource("images/ElectricRod.svg")
EXPORT_DCK_PNG = _ia.ImageAccessor.createForPackageResource("images/exportDck.png")
EXPORT_DCK_SVG = _ia.ImageAccessor.createForPackageResource("images/exportDck.svg")
EXPORT_HYDRAULIC_CONTROL_PNG = _ia.ImageAccessor.createForPackageResource("images/exportHydraulicControl.png")
EXPORT_HYDRAULIC_CONTROL_SVG = _ia.ImageAccessor.createForPackageResource("images/exportHydraulicControl.svg")
EXPORT_HYDRAULICS_PNG = _ia.ImageAccessor.createForPackageResource("images/exportHydraulics.png")
EXPORT_HYDRAULICS_SVG = _ia.ImageAccessor.createForPackageResource("images/exportHydraulics.svg")
EXTERNAL_HX_PNG = _ia.ImageAccessor.createForPackageResource("images/ExternalHx.png")
EXTERNAL_HX_SVG = _ia.ImageAccessor.createForPackageResource("images/ExternalHx.svg")
GEAR_PNG = _ia.ImageAccessor.createForPackageResource("images/gear.png")
GEAR_SVG = _ia.ImageAccessor.createForPackageResource("images/gear.svg")
GENERIC_BLOCK_PNG = _ia.ImageAccessor.createForPackageResource("images/GenericBlock.png")
GENERIC_ITEM_PNG = _ia.ImageAccessor.createForPackageResource("images/GenericItem.png")
GROUND_SOURCE_HX_PNG = _ia.ImageAccessor.createForPackageResource("images/GroundSourceHx.png")
GROUND_SOURCE_HX_SVG = _ia.ImageAccessor.createForPackageResource("images/GroundSourceHx.svg")
HP_PNG = _ia.ImageAccessor.createForPackageResource("images/HP.png")
HP_SVG = _ia.ImageAccessor.createForPackageResource("images/HP.svg")
HP_DOUBLE_DUAL_PNG = _ia.ImageAccessor.createForPackageResource("images/HPDoubleDual.png")
HP_DOUBLE_DUAL_SVG = _ia.ImageAccessor.createForPackageResource("images/HPDoubleDual.svg")
HP_TWO_HX_PNG = _ia.ImageAccessor.createForPackageResource("images/HPTwoHx.png")
HP_TWO_HX_SVG = _ia.ImageAccessor.createForPackageResource("images/HPTwoHx.svg")
ICE_STORAGE_PNG = _ia.ImageAccessor.createForPackageResource("images/IceStorage.png")
ICE_STORAGE_SVG = _ia.ImageAccessor.createForPackageResource("images/IceStorage.svg")
ICE_STORAGE_TWO_HX_PNG = _ia.ImageAccessor.createForPackageResource("images/IceStorageTwoHx.png")
ICE_STORAGE_TWO_HX_SVG = _ia.ImageAccessor.createForPackageResource("images/IceStorageTwoHx.svg")
INBOX_PNG = _ia.ImageAccessor.createForPackageResource("images/inbox.png")
LABEL_TOGGLE_PNG = _ia.ImageAccessor.createForPackageResource("images/labelToggle.png")
OUTBOX_PNG = _ia.ImageAccessor.createForPackageResource("images/outbox.png")
PIT_STORAGE_PNG = _ia.ImageAccessor.createForPackageResource("images/PitStorage.png")
PIT_STORAGE_SVG = _ia.ImageAccessor.createForPackageResource("images/PitStorage.svg")
PROCESS_SIMULATION_PNG = _ia.ImageAccessor.createForPackageResource("images/processSimulation.png")
PROCESS_SIMULATION_SVG = _ia.ImageAccessor.createForPackageResource("images/processSimulation.svg")
PUMP_PNG = _ia.ImageAccessor.createForPackageResource("images/Pump.png")
PUMP_SVG = _ia.ImageAccessor.createForPackageResource("images/Pump.svg")
PV_PNG = _ia.ImageAccessor.createForPackageResource("images/PV.png")
PV_SVG = _ia.ImageAccessor.createForPackageResource("images/PV.svg")
RADIATOR_PNG = _ia.ImageAccessor.createForPackageResource("images/Radiator.png")
RADIATOR_SVG = _ia.ImageAccessor.createForPackageResource("images/Radiator.svg")
ROTATE_LEFT_PNG = _ia.ImageAccessor.createForPackageResource("images/rotate-left.png")
ROTATE_TO_RIGHT_PNG = _ia.ImageAccessor.createForPackageResource("images/rotate-to-right.png")
RUN_MFS_PNG = _ia.ImageAccessor.createForPackageResource("images/runMfs.png")
RUN_MFS_SVG = _ia.ImageAccessor.createForPackageResource("images/runMfs.svg")
RUN_SIMULATION_PNG = _ia.ImageAccessor.createForPackageResource("images/runSimulation.png")
RUN_SIMULATION_SVG = _ia.ImageAccessor.createForPackageResource("images/runSimulation.svg")
STORAGE_TANK_PNG = _ia.ImageAccessor.createForPackageResource("images/StorageTank.PNG")
STORAGE_TANK_SVG = _ia.ImageAccessor.createForPackageResource("images/StorageTank.svg")
TEE_PIECE_PNG = _ia.ImageAccessor.createForPackageResource("images/TeePiece.png")
TEE_PIECE_SVG = _ia.ImageAccessor.createForPackageResource("images/TeePiece.svg")
TRASH_PNG = _ia.ImageAccessor.createForPackageResource("images/trash.png")
T_VENTIL_PNG = _ia.ImageAccessor.createForPackageResource("images/TVentil.png")
T_VENTIL_SVG = _ia.ImageAccessor.createForPackageResource("images/TVentil.svg")
UPDATE_CONFIG_PNG = _ia.ImageAccessor.createForPackageResource("images/updateConfig.png")
UPDATE_CONFIG_SVG = _ia.ImageAccessor.createForPackageResource("images/updateConfig.svg")
VIS_MFS_PNG = _ia.ImageAccessor.createForPackageResource("images/visMfs.png")
VIS_MFS_SVG = _ia.ImageAccessor.createForPackageResource("images/visMfs.svg")
W_TAP_PNG = _ia.ImageAccessor.createForPackageResource("images/WTap.png")
W_TAP_SVG = _ia.ImageAccessor.createForPackageResource("images/WTap.svg")
W_TAP_MAIN_PNG = _ia.ImageAccessor.createForPackageResource("images/WTap_main.png")
W_TAP_MAIN_SVG = _ia.ImageAccessor.createForPackageResource("images/WTap_main.svg")
ZOOM_0_PNG = _ia.ImageAccessor.createForPackageResource("images/zoom-0.png")
ZOOM_IN_PNG = _ia.ImageAccessor.createForPackageResource("images/zoom-in.png")
ZOOM_OUT_PNG = _ia.ImageAccessor.createForPackageResource("images/zoom-out.png")



# pylint: skip-file
# type: ignore

from PyQt5.QtWidgets import QLabel, QLineEdit, QPushButton, QHBoxLayout, QGridLayout, QDialog, QMessageBox


class segmentDlg(QDialog):
    def __init__(self, seg, parent=None):
        super(segmentDlg, self).__init__(parent)
        self.seg = seg
        nameLabel = QLabel("Name:")
        objectLabel = QLabel("Object:" + str(self.seg))
        self.le = QLineEdit(self.seg.parent.displayName)

        self.okButton = QPushButton("OK")
        self.cancelButton = QPushButton("Cancel")

        buttonLayout = QHBoxLayout()
        buttonLayout.addStretch()
        buttonLayout.addWidget(self.okButton)
        buttonLayout.addWidget(self.cancelButton)
        layout = QGridLayout()
        layout.addWidget(nameLabel, 0, 0)
        layout.addWidget(self.le, 0, 1)
        layout.addLayout(buttonLayout, 1, 0, 2, 0)
        # layout.addWidget(objectLabel, 3, 0, 1, 2)  # Only for debug (Why do I need a 3 here instead of a 2 for int:row?)
        self.setLayout(layout)

        self.okButton.clicked.connect(self.acceptedEdit)
        self.cancelButton.clicked.connect(self.cancel)
        self.setWindowTitle("Change connection name")
        self.show()

    def acceptedEdit(self):
        # print("Changing displayName")
        newName = self.le.text()
        if newName.lower() == str(self.seg.parent.displayName).lower():
            self.close()
        elif newName != "" and not self.nameExists(newName):
            self.seg.parent.setDisplayName(newName)
            for segment in self.seg.parent.segments:
                segment.setToolTip(newName)
            self.close()
        elif newName == "":
            msgb = QMessageBox()
            msgb.setText("Please Enter a name!")
            msgb.exec()
        elif self.nameExists(newName):
            msgb = QMessageBox()
            msgb.setText("Name already exist!")
            msgb.exec()

    def cancel(self):
        self.close()

    def nameExists(self, n):
        for t in self.parent().trnsysObj:
            if str(t.displayName).lower() == n.lower():
                return True
        return False



# pylint: skip-file
# type: ignore

import filecmp
import os


class Test_Export(object):
    """
    Just a class that contains the method for testing.
    No need to declare or pass anything.
    Parameters should be passed into the individual functions to avoid unnecessary instantiotion
    of class.
    """

    testPassed = True

    def retrieveFiles(self, exportedFilepath, originalFilePath, exportedFileList, originalFileList):
        """

        Parameters
        ----------
        exportedFilepath : Folder where files are exported to
        originalFilePath : Folder containing the reference files
        exportedFileList : List of files inside the exported folder
        originalFileList : List of files inside the reference folder

        1.Access the exported folder
        2.Retrieve all exported files and add into a list
        3.Access the reference folder
        4.Retrieve all reference files and add into a list
        5.Sort the two lists

        Returns
        -------

        """
        for efiles in os.listdir(exportedFilepath):
            if efiles != ".keepMe":
                # exportedFile = exportedFilepath + efiles
                exportedFile = os.path.join(exportedFilepath, efiles)
                if exportedFile not in exportedFileList:
                    exportedFileList.append(exportedFile)

        for ofiles in os.listdir(originalFilePath):
            # originalFile = originalFilePath + ofiles
            originalFile = os.path.join(originalFilePath, ofiles)
            if originalFile not in originalFileList:
                originalFileList.append(originalFile)

        self.sortList(exportedFileList, originalFileList)

    def checkFileExists(self, exportedFile, originalFileList):
        """

        Parameters
        ----------
        exportedFile : ONE FILE from the export_test folder
        originalFileList : list of files inside the reference folder

        1.Check if the individual files inside the export_test folder can be found
        inside the reference folder as well.
        2.If found, return true.
        3.Else, return false.

        Returns
        -------

        """
        i = 0
        exportedFile = exportedFile.split("\\")
        while i < len(originalFileList):
            referenceFile = originalFileList[i].split("\\")
            if exportedFile[-1] == referenceFile[-1]:
                return True
            i += 1
        return False

    def checkFiles(self, exportedFileList, originalFileList):
        """

        Parameters
        ----------
        exportedFileList : All files inside the export_test folder
        originalFileList : All files inside the reference folder

        1.Check each exported file against the reference file of the same name
        2.If two files of the same name from both folders are found, check their contents
        3.If the contents are not the same, append the file index(order inside the folder) into fileErrorList
          and set testPassed to False
        4.Return fileErrorList and testPassed attribute

        Returns
        -------

        """

        i = 0
        fileErrorList = []
        found = False
        while i < len(exportedFileList):
            j = 0
            while j < len(originalFileList):
                exportedFile = exportedFileList[i].split("\\")[-1]
                referencefile = originalFileList[j].split("\\")[-1]
                # print('this is exported :' + exportedFile + '\n')
                # print('this is reference :' + referencefile + '\n')
                if exportedFile != referencefile:
                    j += 1
                else:
                    found = True
                    break
            if found:
                if not filecmp.cmp(exportedFileList[i], originalFileList[j], shallow=False):
                    # todo : can maybe simplify this part is not using random number for pump power
                    fileOne = open(exportedFileList[i])
                    fileTwo = open(originalFileList[j])

                    fileOneLine = fileOne.readline()
                    fileTwoLine = fileTwo.readline()

                    while fileOneLine:
                        if fileOneLine != fileTwoLine:
                            if fileOneLine.split(" = ")[0][:5] != "MfrPu":
                                self.testPassed = False
                                fileErrorList.append(i)
                                break

                        fileOneLine = fileOne.readline()
                        fileTwoLine = fileTwo.readline()
                    # break
            i += 1
        return fileErrorList, self.testPassed
        # return i, self.testPassed

    def deleteFiles(self, fileDirectory):
        """

        Parameters
        ----------
        fileDirectory : A specified file directory

        1.Delete every files inside the specified file directory

        Returns
        -------

        """
        for dfiles in os.listdir(fileDirectory):
            # deleteFiles = fileDirectory + dfiles
            deleteFiles = os.path.join(fileDirectory, dfiles)
            if dfiles != ".keepMe":
                os.remove(deleteFiles)

    def showDifference(self, file1, file2):
        """

        Parameters
        ----------
        file1 : exported file
        file2 : reference file

        1.Read a line from exported file and reference file
        2.Compare the two lines, if they are identical, go to next line.
        3.Else, Append the lines with their line number into two separate list.
        4.Close the files and return the two lists.

        Returns
        -------

        """
        line = 1
        fileOne = open(file1)
        fileTwo = open(file2)

        file1List = []
        file2List = []

        fileOneLine = fileOne.readline()
        fileTwoLine = fileTwo.readline()

        while fileOneLine:
            if fileOneLine != fileTwoLine:
                print("Error found at line %d\n" % line)
                print("Exported file: %s \nReference file: %s \n" % (fileOneLine, fileTwoLine))
                if fileOneLine.split(" = ")[0][:5] != "MfrPu":
                    file1Str = str(line) + ":" + fileOneLine
                    file2Str = str(line) + ":" + fileTwoLine
                    file1List.append(file1Str)
                    file2List.append(file2Str)
            fileOneLine = fileOne.readline()
            fileTwoLine = fileTwo.readline()
            line += 1

        # print("File 1 list:")
        # print(file1List)
        # print("\nFile 2 List:")
        # print(file2List)

        fileOne.close()
        fileTwo.close()

        return file1List, file2List

    def sortList(self, list1, list2):
        list1.sort()
        list2.sort()



import typing as _tp


class Cancelled:
    pass


CANCELLED = Cancelled()
_T = _tp.TypeVar("_T")
MaybeCancelled = _tp.Union[_T, Cancelled]


def isCancelled(maybeCancelled: MaybeCancelled[_T]) -> bool:
    if maybeCancelled == CANCELLED:
        return True

    if isinstance(maybeCancelled, Cancelled):
        raise AssertionError(f"Don't instantiate {Cancelled} but use the variable `CANCELLED' instead.")

    return False


def value(maybeCancelled: MaybeCancelled[_T]) -> _T:
    if isCancelled(maybeCancelled):
        raise ValueError("Value was cancelled.")

    return _tp.cast(_T, maybeCancelled)



# pylint: skip-file
# type: ignore

import pytrnsys.trnsys_util.buildTrnsysDeck as build
import numpy as num
import os
import pytrnsys.trnsys_util.readConfigTrnsys as readConfig
import pytrnsys.utils.log as log
import logging

logger = logging.getLogger("root")


class buildDck:
    def __init__(self, pathConfig, name="pytrnsysRun", configFile=None, runPath=None):

        self.pathConfig = pathConfig
        self.path = pathConfig

        self.defaultInputs()
        self.cmds = []

        self.readConfig(self.pathConfig, "run.config")
        self.getConfig()

        self.nameBase = self.inputs["nameRef"]

        self.buildTrnsysDeck()

    def defaultInputs(self):

        self.inputs = {}
        self.inputs["ignoreOnlinePlotter"] = False
        self.inputs["removePopUpWindow"] = False

        self.inputs["checkDeck"] = True
        self.inputs["reduceCpu"] = 0
        self.inputs["combineAllCases"] = True
        self.inputs["parseFileCreated"] = True
        self.inputs["HOME$"] = None
        self.inputs["trnsysVersion"] = "TRNSYS_EXE"
        self.inputs["trnsysExePath"] = "enviromentalVariable"
        self.inputs["copyBuildingData"] = False  # activate when Type 55 is used or change the path to the source
        self.inputs["addResultsFolder"] = False
        self.inputs["rerunFailedCases"] = False
        self.inputs["scaling"] = False
        self.inputs["doAutoUnitNumbering"] = True
        self.inputs["addAutomaticEnergyBalance"] = True
        self.inputs["generateUnitTypesUsed"] = True
        self.inputs["runCases"] = True
        self.inputs["runType"] = "runFromConfig"
        self.inputs["outputLevel"] = "INFO"

        self.overwriteForcedByUser = False

        self.variablesOutput = []

    def buildTrnsysDeck(self):
        """
        It builds a TRNSYS Deck from a listDdck with pathDdck using the BuildingTrnsysDeck Class.
        it reads the Deck list and writes a deck file. Afterwards it checks that the deck looks fine

        """
        deckExplanation = []
        deckExplanation.append("! ** New deck built from list of ddcks. **\n")
        deck = build.BuildTrnsysDeck(self.path, self.nameBase, self.listDdck)
        deck.readDeckList(
            self.pathConfig,
            doAutoUnitNumbering=self.inputs["doAutoUnitNumbering"],
            dictPaths=self.dictDdckPaths,
            replaceLineList=self.replaceLines,
        )

        deck.overwriteForcedByUser = self.overwriteForcedByUser
        deck.writeDeck(addedLines=deckExplanation)
        if deck.abortedByUser:
            return
        self.overwriteForcedByUser = deck.overwriteForcedByUser

        deck.checkTrnsysDeck(deck.nameDeck, check=self.inputs["checkDeck"])

        if self.inputs["generateUnitTypesUsed"] == True:

            deck.saveUnitTypeFile()

        if self.inputs["addAutomaticEnergyBalance"] == True:
            deck.automaticEnegyBalanceStaff()
            deck.writeDeck()  # Deck rewritten with added printer

        return deck.nameDeck

    def addParametricVariations(self, variations):
        """
        it fills a variableOutput with a list of all variations to run
        format <class 'list'>: [['Ac', 'AcollAp', 1.5, 2.0, 1.5, 2.0], ['Vice', 'VIceS', 0.3, 0.3, 0.4, 0.4]]

        Parameters
        ----------
        variations : list of list
            list object containing the variations to be used.

        Returns
        -------

        """

        if self.inputs["combineAllCases"] == True:

            labels = []
            values = []
            for i, row in enumerate(variations):
                labels.append(row[:2])
                values.append(row[2:])

            value_permutations = num.array(num.meshgrid(*values), dtype=object).reshape(len(variations), -1)
            result = num.concatenate((labels, value_permutations), axis=1)
            self.variablesOutput = result.tolist()

        else:
            nVariations = len(variations)
            sizeOneVariation = len(variations[0]) - 2
            for n in range(len(variations)):
                sizeCase = len(variations[n]) - 2
                if sizeCase != sizeOneVariation:
                    raise ValueError(
                        "for combineAllCases=False all variations must have same lenght :%d case n:%d has a lenght of :%d"
                        % (sizeOneVariation, n + 1, sizeCase)
                    )

            self.variablesOutput = variations

    def readConfig(self, path, name, parseFileCreated=False):

        """
        It reads the config file used for running TRNSYS and loads the self.inputs dictionary.
        It also loads the readed lines into self.lines
        """
        tool = readConfig.ReadConfigTrnsys()

        self.lines = tool.readFile(path, name, self.inputs, parseFileCreated=parseFileCreated, controlDataType=False)
        # logger = log.setup_custom_logger('root', self.inputs['outputLevel'])
        # stop propagting to root logger
        # logger.propagate = False
        if "pathBaseSimulations" in self.inputs:
            self.path = self.inputs["pathBaseSimulations"]
        if self.inputs["addResultsFolder"] == False:
            pass
        else:
            self.path = os.path.join(self.path, self.inputs["addResultsFolder"])

            if not os.path.isdir(self.path):
                os.mkdir(self.path)

    def getConfig(self):
        """
        Reads the config file.

        Parameters
        ----------

        Returns
        -------

        """
        self.variation = []  # parametric studies
        self.parDeck = []  # fixed values changed in all simulations
        self.listDdck = []
        self.parameters = {}  # deck parameters fixed for all simulations
        self.listFit = {}
        self.listFitObs = []
        self.listDdckPaths = set()
        self.dictDdckPaths = {}
        self.caseDict = {}
        self.sourceFilesToChange = []
        self.sinkFilesToChange = []
        self.foldersForDDckVariation = []
        self.replaceLines = []

        for line in self.lines:

            splitLine = line.split()

            if splitLine[0] == "variation":
                variation = []
                for i in range(len(splitLine)):
                    if i == 0:
                        pass
                    elif i <= 2:
                        variation.append(splitLine[i])
                    else:
                        try:
                            variation.append(float(splitLine[i]))
                        except:
                            variation.append(splitLine[i])

                self.variation.append(variation)

            elif splitLine[0] == "deck":

                if splitLine[2] == "string":
                    self.parameters[splitLine[1]] = splitLine[3]
                else:
                    if splitLine[2].isdigit():
                        self.parameters[splitLine[1]] = float(splitLine[2])
                    else:
                        self.parameters[splitLine[1]] = splitLine[2]

            elif splitLine[0] == "replace":

                splitString = line.split('$"')

                oldString = splitString[1].split('"')[0]
                newString = splitString[2].split('"')[0]

                self.replaceLines.append((oldString, newString))

            elif splitLine[0] == "changeDDckFile":
                self.sourceFilesToChange.append(splitLine[1])
                sinkFilesToChange = []
                for i in range(len(splitLine)):
                    if i < 2:
                        pass
                    else:
                        sinkFilesToChange.append(splitLine[i])
                self.sinkFilesToChange.append(sinkFilesToChange)

            elif splitLine[0] == "addDDckFolder":
                for i in range(len(splitLine)):
                    if i > 0:
                        self.foldersForDDckVariation.append(splitLine[i])

            elif splitLine[0] == "fit":
                self.listFit[splitLine[1]] = [splitLine[2], splitLine[3], splitLine[4]]
            elif splitLine[0] == "case":
                self.listFit[splitLine[1]] = splitLine[2:]
            elif splitLine[0] == "fitobs":
                self.listFitObs.append(splitLine[1])

            elif splitLine[0] in self.inputs.keys():
                fullPath = os.path.join(self.inputs[splitLine[0]], splitLine[1])
                self.listDdck.append(fullPath)
                self.listDdckPaths.add(self.inputs[splitLine[0]])
                self.dictDdckPaths[fullPath] = self.inputs[splitLine[0]]
            else:

                pass

        if len(self.variation) > 0:
            self.addParametricVariations(self.variation)
            self.variationsUsed = True
        else:
            self.variationsUsed = False

        if len(self.sourceFilesToChange) > 0:
            self.changeDDckFilesUsed = True
        else:
            self.changeDDckFilesUsed = False

        if len(self.foldersForDDckVariation) > 0:
            self.foldersForDDckVariationUsed = True
        else:
            self.foldersForDDckVariationUsed = False



# pylint: skip-file
# type: ignore

import os
import shutil
import typing as _tp

from PyQt5.QtCore import QSize
from PyQt5.QtGui import QPixmap
from PyQt5.QtWidgets import QTreeView

import trnsysGUI.images as _img
from trnsysGUI.BlockItemFourPorts import BlockItemFourPorts
from trnsysGUI.MyQFileSystemModel import MyQFileSystemModel
from trnsysGUI.MyQTreeView import MyQTreeView


class ExternalHx(BlockItemFourPorts):
    def __init__(self, trnsysType, parent, **kwargs):
        super().__init__(trnsysType, parent, **kwargs)

        self.loadedFiles = []

        self.w = 80
        self.h = 120

        self.changeSize()
        self.addTree()

    def _getImageAccessor(self) -> _tp.Optional[_img.ImageAccessor]:
        return _img.EXTERNAL_HX_SVG

    def changeSize(self):
        w = self.w
        h = self.h

        """ Resize block function """
        delta = 20

        # Limit the block size:
        if h < 20:
            h = 20
        if w < 40:
            w = 40
        # center label:
        rect = self.label.boundingRect()
        lw, lh = rect.width(), rect.height()
        lx = (w - lw) / 2
        self.label.setPos(lx, h)

        self.origInputsPos = [[0, delta], [w, delta]]
        self.origOutputsPos = [[0, h - delta], [w, h - delta]]
        self.inputs[0].setPos(self.origInputsPos[0][0], self.origInputsPos[0][1])
        self.inputs[1].setPos(self.origInputsPos[1][0], self.origInputsPos[1][1])
        self.outputs[0].setPos(self.origOutputsPos[0][0], self.origOutputsPos[0][1])
        self.outputs[1].setPos(self.origOutputsPos[1][0], self.origOutputsPos[1][1])

        self.updateFlipStateH(self.flippedH)
        self.updateFlipStateV(self.flippedV)

        self.inputs[0].side = (self.rotationN + 2 * self.flippedH) % 4
        self.inputs[1].side = (self.rotationN + 2 - 2 * self.flippedH) % 4

        self.outputs[0].side = (self.rotationN + 2 * self.flippedH) % 4
        self.outputs[1].side = (self.rotationN + 2 - 2 * self.flippedH) % 4

        return w, h

    def addTree(self):
        """
        When a blockitem is added to the main window.
        A file explorer for that item is added to the right of the main window by calling this method
        """
        self.logger.debug(self.parent.parent())
        pathName = self.displayName
        if self.parent.parent().projectPath == "":
            # self.path = os.path.dirname(__file__)
            # self.path = os.path.join(self.path, 'default')
            self.path = self.parent.parent().projectFolder
            # now = datetime.now()
            # self.fileName = now.strftime("%Y%m%d%H%M%S")
            # self.path = os.path.join(self.path, self.fileName)
        else:
            self.path = self.parent.parent().projectPath
        self.path = os.path.join(self.path, "ddck")
        self.path = os.path.join(self.path, pathName)
        if not os.path.exists(self.path):
            os.makedirs(self.path)

        self.model = MyQFileSystemModel()
        self.model.setRootPath(self.path)
        self.model.setName(self.displayName)
        self.tree = MyQTreeView(self.model, self)
        self.tree.setModel(self.model)
        self.tree.setRootIndex(self.model.index(self.path))
        self.tree.setObjectName("%sTree" % self.displayName)
        for i in range(1, self.model.columnCount() - 1):
            self.tree.hideColumn(i)
        self.tree.setMinimumHeight(200)
        self.tree.setSortingEnabled(True)
        self.parent.parent().splitter.addWidget(self.tree)

    # def loadFile(self, file):
    #     filePath = self.parent.parent().projectPath
    #     msgB = QMessageBox()
    #     if filePath == '':
    #         msgB.setText("Please select a project path before loading!")
    #         msgB.exec_()
    #     else:
    #         self.logger.debug("file loaded into %s" % filePath)
    #         shutil.copy(file, filePath)

    def updateTreePath(self, path):
        """
        When the user chooses the project path for the file explorers, this method is called
        to update the root path.
        """
        pathName = self.displayName
        self.path = os.path.join(path, "ddck")
        self.path = os.path.join(self.path, pathName)
        if not os.path.exists(self.path):
            os.makedirs(self.path)
        self.model.setRootPath(self.path)
        self.tree.setRootIndex(self.model.index(self.path))

    def deleteBlock(self):
        """
        Overridden method to also delete folder
        """
        self.logger.debug("Block " + str(self) + " is deleting itself (" + self.displayName + ")")
        self.deleteConns()
        # self.logger.debug("self.parent.parent" + str(self.parent.parent()))
        self.parent.parent().trnsysObj.remove(self)
        self.logger.debug("deleting block " + str(self) + self.displayName)
        # self.logger.debug("self.scene is" + str(self.parent.scene()))
        self.parent.scene().removeItem(self)
        widgetToRemove = self.parent.parent().findChild(QTreeView, self.displayName + "Tree")
        shutil.rmtree(self.path)
        self.deleteLoadedFile()
        try:
            widgetToRemove.hide()
        except AttributeError:
            self.logger.debug("Widget doesnt exist!")
        else:
            self.logger.debug("Deleted widget")
        del self

    def setName(self, newName):
        """
        Overridden method to also change folder name
        """
        self.displayName = newName
        self.label.setPlainText(newName)
        self.model.setName(self.displayName)
        self.tree.setObjectName("%sTree" % self.displayName)
        self.logger.debug(os.path.dirname(self.path))
        # destPath = str(os.path.dirname(self.path))+'\\ExternalHx_'+self.displayName
        destPath = os.path.join(os.path.split(self.path)[0], self.displayName)
        if os.path.exists(self.path):
            os.rename(self.path, destPath)
            self.path = destPath
            self.logger.debug(self.path)



# pylint: skip-file
# type: ignore

import sys
import re

from PyQt5.QtWidgets import QDialog, QLabel, QHBoxLayout, QPushButton, QLineEdit, QGridLayout, QListWidget, QRadioButton


class DeepInspector(QDialog):
    def __init__(self, parent):
        super(DeepInspector, self).__init__(parent)

        self.stdout = sys.stdout
        self.messages = []

        self.editor = parent
        self.currentObj = parent

        funcLabel = QLabel("Enter function call:")
        self.le = QLineEdit()

        self.okButton = QPushButton("Execute")
        self.cancelButton = QPushButton("Cancel")

        self.listW = QListWidget()

        radioButtonLayout = QHBoxLayout()
        self.rMeth = QRadioButton("Method")
        self.rAttr = QRadioButton("Attr")
        radioButtonLayout.addWidget(self.rMeth)
        radioButtonLayout.addWidget(self.rAttr)

        buttonLayout = QHBoxLayout()
        buttonLayout.addStretch()
        buttonLayout.addWidget(self.okButton)
        buttonLayout.addWidget(self.cancelButton)
        layout = QGridLayout()
        layout.addWidget(funcLabel, 0, 0)
        layout.addWidget(self.le, 1, 0)
        layout.addWidget(self.listW, 2, 0)
        layout.addLayout(radioButtonLayout, 3, 0, 2, 0)
        layout.addLayout(buttonLayout, 5, 0, 2, 0)
        self.setLayout(layout)

        self.okButton.clicked.connect(self.acceptedEdit)
        self.cancelButton.clicked.connect(self.cancel)
        self.setWindowTitle("Diagram Properties")
        self.show()

    def acceptedEdit(self):
        if not (self.rMeth.isChecked() or self.rAttr.isChecked()):
            print("No button checked")
        else:
            t = self.le.text()
            # b = False
            # if "(" in t:
            #     b = True
            if self.rMeth.isChecked():
                funcStr = re.findall(".{0,}\(", t)[0][:-1]
                argStr = re.findall("\(.{0,}\)", t)[0][1:-1]
                print(funcStr + ", " + argStr)
                if argStr == "":
                    self.executeMeth((funcStr))
                else:
                    self.executeMeth(funcStr, argStr)
            else:
                funcStr = t
                argStr = None
                self.currentObj = self.getAtr(funcStr)

            # self.stopLog()
            print("now in console should appear: " + str(self.messages))
            self.listW.addItem(funcStr)

    def cancel(self):
        self.close()

    def executeMeth(self, meth, *args):
        res = getattr(self.currentObj, meth)(*args)
        # if type(res) is not int and type(res) is not list:
        print(type(res))
        self.currentObj = res

    def getAtr(self, name):
        return getattr(self.currentObj, name)

    def startLog(self):
        sys.stdout = self

    def stopLog(self):
        sys.stdout = self.stdout

    def write(self, text):
        self.messages.append(text)



# pylint: skip-file
# type: ignore

class Node(object):
    def __init__(self, parent=None, prevNode=None, nextNode=None):
        """
        Nodes
        can
        have as parent
        1)Connection (then it is at a PortItem)
        2)CornerItem

        Parameters
        ----------
        parent
        prevNode
        nextNode
        """

        self.prevNode = prevNode
        self.nextNode = nextNode
        self.parent = parent

    def nextN(self):
        return self.nextNode

    def prevN(self):
        return self.prevNode

    def setPrev(self, prevNode):
        self.prevNode = prevNode

    def setNext(self, nextNode):
        self.nextNode = nextNode

    def setParent(self, parent):
        self.parent = parent

    def firstNode(self):
        # Recursion
        # if self.prevNode is not None:
        #     return self.prevNode.beginNode()
        # else:
        #     return self

        # Should do the same:
        res = self
        while res.prevNode is not None:
            res = res.prevNode

        return res

    def lastNode(self):
        # Recursion
        # if self.nextNode is not None:
        #     return self.nextnode.lastNode()
        # else:
        #     return self

        # Should do the same:
        res = self
        while res.nextN() is not None:
            res = res.nextN()
            # print("In traversal... at node " + str(self) + " has parent " + str(self.parent))
        return res

    def countNodes(self):
        res = 1
        n = self.firstNode(self)
        while res.nextN() is not None:
            res += 1
            n = n.nextN()
        print("This connection has " + str(res) + "nodes")
        return res



# pylint: skip-file
# type: ignore

from PyQt5.QtWidgets import QDialog, QDialogButtonBox, QVBoxLayout, QLabel


class closeDlg(QDialog):
    """
    Not used currently. Keep for future sake.
    Used to prompt user whether to save their current template
    """

    def __init__(self, *args, **kwargs):
        super(closeDlg, self).__init__(*args, **kwargs)

        self.closeBool = False

        self.setWindowTitle("Closing!")
        self.dlgMessage = QLabel()
        self.strMessage = "Do you want to retain the current template for when you next open the application?"
        self.dlgMessage.setText(self.strMessage)

        QBtn = QDialogButtonBox.Ok | QDialogButtonBox.Cancel

        self.buttonBox = QDialogButtonBox(QBtn)
        self.buttonBox.accepted.connect(self.acceptExport)
        self.buttonBox.rejected.connect(self.reject)

        self.layout = QVBoxLayout()
        self.layout.addWidget(self.dlgMessage)
        self.layout.addWidget(self.buttonBox)
        self.setLayout(self.layout)
        self.exec_()

    def acceptExport(self):
        self.closeBool = True
        self.accept()



# pylint: skip-file
# type: ignore

import typing as _tp

from PyQt5.QtCore import QSize
from PyQt5.QtGui import QPixmap

from trnsysGUI.BlockItem import BlockItem
from trnsysGUI.PortItem import PortItem
import trnsysGUI.images as _img


class TeePiece(BlockItem):
    def __init__(self, trnsysType, parent, **kwargs):
        super(TeePiece, self).__init__(trnsysType, parent, **kwargs)

        self.w = 40
        self.h = 40

        self.typeNumber = 2

        self.inputs.append(PortItem("i", 0, self))
        self.inputs.append(PortItem("i", 2, self))
        self.outputs.append(PortItem("o", 1, self))

        self.changeSize()

        # self.addTree()

    def _getImageAccessor(self) -> _tp.Optional[_img.ImageAccessor]:
        return _img.TEE_PIECE_SVG

    def changeSize(self):
        w = self.w
        h = self.h

        delta = 20
        # Limit the block size:
        if h < 20:
            h = 20
        if w < 40:
            w = 40

        # center label:
        rect = self.label.boundingRect()
        lw, lh = rect.width(), rect.height()
        lx = (w - lw) / 2

        deltaH = self.h / 8

        self.label.setPos(lx, h - self.flippedV * (h + h / 2))

        self.origInputsPos = [[0, delta], [w, delta]]
        self.origOutputsPos = [[delta, 0]]
        self.inputs[0].setPos(self.origInputsPos[0][0], self.origInputsPos[0][1])
        self.inputs[1].setPos(self.origInputsPos[1][0], self.origInputsPos[1][1])
        self.outputs[0].setPos(self.origOutputsPos[0][0], self.origOutputsPos[0][1])

        self.updateFlipStateH(self.flippedH)
        self.updateFlipStateV(self.flippedV)

        self.inputs[0].side = (self.rotationN + 2 * self.flippedH) % 4
        self.inputs[1].side = (self.rotationN + 2 - 2 * self.flippedH) % 4
        self.outputs[0].side = (self.rotationN + 1 - 1 * self.flippedH) % 4

        return w, h

    def exportParametersFlowSolver(self, descConnLength):
        temp = ""
        for i in self.inputs:
            for c in i.connectionList:
                temp = temp + str(c.trnsysId) + " "
                self.trnsysConn.append(c)

        for o in self.outputs:
            for c in o.connectionList:
                temp = temp + str(c.trnsysId) + " "
                self.trnsysConn.append(c)

        temp += str(self.typeNumber)
        temp += " " * (descConnLength - len(temp))
        self.exportConnsString = temp

        f = temp + "!" + str(self.trnsysId) + " : " + str(self.displayName) + "\n"

        return f

    def exportOutputsFlowSolver(self, prefix, abc, equationNumber, simulationUnit):
        if self.isVisible():
            tot = ""
            for i in range(0, 3):
                temp = (
                    prefix
                    + self.displayName
                    + "_"
                    + abc[i]
                    + "=["
                    + str(simulationUnit)
                    + ","
                    + str(equationNumber)
                    + "]\n"
                )
                tot += temp
                self.exportEquations.append(temp)
                # nEqUsed += 1  # DC
                equationNumber += 1  # DC-ERR
            return tot, equationNumber, 3
        else:
            return "", equationNumber + 3, 0

    def exportPipeAndTeeTypesForTemp(self, startingUnit):
        if self.isVisible():
            f = ""
            unitNumber = startingUnit
            tNr = 929  # Temperature calculation from a tee-piece

            unitText = ""
            ambientT = 20

            equationConstant = 1

            unitText += "UNIT " + str(unitNumber) + " TYPE " + str(tNr) + "\n"
            unitText += "!" + self.displayName + "\n"
            unitText += "PARAMETERS 0\n"
            unitText += "INPUTS 6\n"

            for s in self.exportEquations:
                unitText += s[0 : s.find("=")] + "\n"

            for it in self.trnsysConn:
                unitText += "T" + it.displayName + "\n"

            unitText += "***Initial values\n"
            unitText += 3 * "0 " + 3 * (str(ambientT) + " ") + "\n"

            unitText += "EQUATIONS 1\n"
            unitText += "T" + self.displayName + "= [" + str(unitNumber) + "," + str(equationConstant) + "]\n"

            unitNumber += 1
            f += unitText + "\n"

            return f, unitNumber
        else:
            return "", startingUnit



# pylint: skip-file
# type: ignore

import sys

from PyQt5 import QtCore
from PyQt5.QtCore import QRectF, Qt, QPointF
from PyQt5.QtGui import QColor, QBrush, QCursor, QPen
from PyQt5.QtWidgets import QGraphicsEllipseItem, QMenu


class PortItem(QGraphicsEllipseItem):
    def __init__(self, name, side, parent):
        self.parent = parent

        self.logger = parent.logger

        self.name = name
        self.side = side
        self.createdAtSide = side
        self.posCallbacks = []
        self.connectionList = []
        self.id = self.parent.parent.parent().idGen.getID()

        self.color = "white"
        self.ashColorR = QColor(239, 57, 75)
        self.ashColorB = QColor(20, 83, 245)

        QGraphicsEllipseItem.__init__(self, QRectF(-4, -4, 7.0, 7.0), parent)

        self.innerCircle = QGraphicsEllipseItem(-4, -4, 6, 6, self)
        self.innerCircle.setPen(QPen(QColor(0, 0, 0, 0), 0))

        self.visibleColor = QColor(0, 0, 0)

        # This if is only for input/output having different colors
        if name == "i":
            color = QColor(self.ashColorR)
            # self.innerCircle.setBrush(QColor(self.ashColorR))
            # self.innerCircle.setBrush(QColor(0, 0, 0))
            self.innerCircle.setBrush(self.visibleColor)
        if name == "o":
            color = QColor(self.ashColorB)
            # self.innerCircle.setBrush(QColor(self.ashColorB))
            # self.innerCircle.setBrush(QColor(0, 0, 0))
            self.innerCircle.setBrush(self.visibleColor)

        self.setCursor(QCursor(QtCore.Qt.CrossCursor))
        # self.setCursor(QCursor(QtCore.Qt.PointingHandCursor))

        # The Port itself is larger than only the innerCircle
        self.setBrush(QBrush(QtCore.Qt.white))

        # Hacky fix for no border of ellipse
        p1 = QPen(QColor(0, 0, 0, 0), 0)
        self.setPen(p1)

        self.setFlag(self.ItemSendsScenePositionChanges, True)
        self.setAcceptHoverEvents(True)

        # QUndo framework related
        self.savePos = None
        self.savedPos = False

    def setParent(self, p):
        self.parent = p
        self.setParentItem(p)

    def itemChange(self, change, value):
        # TODO : here to merge segments when moving blockitems
        if (
            self.parent.parent.parent().moveDirectPorts
            and hasattr(self.parent, "heatExchangers")
            and change == self.ItemPositionChange
        ):
            if not self.savePos is None:
                self.logger.debug("val is " + str(value))
                value.setY(max(value.y(), 0))
                value.setY(min(value.y(), self.parent.h))

                return QPointF(self.savePos.x(), value.y())

        if change == self.ItemScenePositionHasChanged and self.parent.parent.parent().editorMode == 0:
            self.logger.debug("editor mode = 0")
            for conn in self.connectionList:
                conn.positionLabel()

                if conn.fromPort is self:
                    # self.logger.debug("This port is the starting port of connection")
                    # nextNodeInConn = conn.startNode.nextN()
                    # self.logger.debug("Type of parent is " + str(type(nextNodeInConn.parent)))

                    e = conn.segments[0]
                    e.setLine(self.scenePos().x(), self.scenePos().y(), e.line().p2().x(), e.line().p2().y())

                if conn.toPort is self:
                    # self.logger.debug("This port is the ending port of connection")
                    # nextNodeInConn = conn.endNode.prevN()
                    # self.logger.debug("nextnodeinconn is " + str(conn.endNode.prevN()))
                    # self.logger.debug("nextNode is " + str(nextNodeInConn))
                    # self.logger.debug("Type of parent is " + str(type(nextNodeInConn.parent)))

                    # New
                    e = conn.segments[-1]
                    e.setLine(e.line().p1().x(), e.line().p1().y(), self.scenePos().x(), self.scenePos().y())

                # global pasting
                if not self.parent.parent.parent().pasting:
                    for s in conn.segments:
                        s.updateGrad()

        if change == self.ItemScenePositionHasChanged and self.parent.parent.parent().editorMode == 1:
            for conn in self.connectionList:
                # Update position of connection label
                conn.positionLabel()

                if conn.fromPort is self:
                    if (self.createdAtSide != 1 and self.createdAtSide != 3) or not conn.segments[0].isVertical():
                        if len(conn.getCorners()) > 0 and len(conn.segments) > 0:
                            self.logger.debug("inside here")
                            cor = conn.getCorners()[0]
                            cor.setPos(cor.pos().x(), self.scenePos().y())

                            seg = conn.segments[0]  # first segment
                            seg.setLine(
                                self.scenePos().x(), self.scenePos().y(), cor.scenePos().x() + 0.6, cor.scenePos().y()
                            )
                            if len(conn.segments) > 2:
                                verSeg = conn.segments[1]
                                nextSeg = conn.segments[2]
                                if nextSeg.isHorizontal() and seg.isHorizontal():
                                    if (
                                        int(seg.endNode.parent.pos().y() - 0)
                                        <= int(nextSeg.line().p2().y())
                                        <= int(seg.endNode.parent.pos().y() + 0)
                                    ):
                                        self.logger.debug("both segments are horizontal from fromport")
                                        # verSeg.deleteSegment()
                                        self.hideCorners(conn)
                                        verSeg.setVisible(False)
                                    else:
                                        self.showCorners(conn)
                                        verSeg.setVisible(True)
                    else:
                        self.logger.debug("inside else")
                        if len(conn.getCorners()) > 0 and len(conn.segments) > 0:
                            cor = conn.getCorners()[0]
                            cor.setPos(self.scenePos().x(), cor.pos().y())

                            seg = conn.segments[0]  # first segment
                            seg.setLine(self.scenePos().x(), self.scenePos().y(), cor.scenePos().x(), cor.scenePos().y())

                elif conn.toPort is self:
                    if (conn.fromPort.createdAtSide != 1 and conn.fromPort.createdAtSide != 3) or not conn.segments[
                        0
                    ].isVertical():
                        if len(conn.getCorners()) > 0 and len(conn.segments) > 0:
                            cor = conn.getCorners()[-1]
                            cor.setPos(cor.pos().x(), self.scenePos().y())

                            seg = conn.segments[-1]
                            seg.setLine(
                                self.scenePos().x(),
                                self.scenePos().y(),
                                cor.scenePos().x() + 0.6,
                                cor.scenePos().y() + 0.6,
                            )
                            if len(conn.segments) > 2:
                                verSeg = conn.segments[-2]
                                nextSeg = conn.segments[-3]
                                if nextSeg.isHorizontal() and seg.isHorizontal():
                                    if (
                                        int(nextSeg.endNode.parent.pos().y() - 0)
                                        <= int(seg.line().p2().y())
                                        <= int(nextSeg.endNode.parent.pos().y() + 0)
                                    ):
                                        self.logger.debug("both segments are horizontal from toport")
                                        self.hideCorners(conn)
                                        verSeg.setVisible(False)
                                    else:
                                        self.showCorners(conn)
                                        verSeg.setVisible(True)
                    else:
                        if len(conn.getCorners()) == 1 and len(conn.segments) > 0:
                            cor = conn.getCorners()[-1]
                            cor.setPos(cor.pos().x(), self.scenePos().y())
                            self.logger.debug("Inside 2nd")

                            seg = conn.segments[-1]  # last segment
                            seg.setLine(self.scenePos().x(), self.scenePos().y(), cor.scenePos().x(), cor.scenePos().y())
                        elif len(conn.getCorners()) == 2 and len(conn.segments) > 0:
                            cor = conn.getCorners()[-1]
                            cor.setPos(self.scenePos().x(), cor.pos().y())
                            self.logger.debug("Inside 3rd")

                            seg = conn.segments[-1]  # last segment
                            seg.setLine(self.scenePos().x(), self.scenePos().y(), cor.scenePos().x(), cor.scenePos().y())

                else:
                    self.logger.debug("Error: In Mode 1, moving a portItem, portItem is neither from nor toPort")

        if change == self.ItemScenePositionHasChanged:
            for cb in self.posCallbacks:
                cb(value)
            return value
        return super(PortItem, self).itemChange(change, value)

    def mousePressEvent(self, event):
        if self.parent.parent.parent().moveDirectPorts and hasattr(self.parent, "heatExchangers"):
            self.setFlag(self.ItemIsMovable)
            self.savePos = self.pos()
        else:
            self.setFlag(self.ItemIsMovable, False)
            self.scene().parent().startConnection(self)

    def hoverEnterEvent(self, event):
        # self.logger.debug("Hovering")

        self.logger.debug(self.parent)

        self.setRect(-4, -4, 10, 10)
        self.innerCircle.setRect(-4, -4, 10, 10)
        if self.name == "i":
            # self.setBrush(Qt.red)
            # self.innerCircle.setBrush(Qt.red)
            # self.innerCircle.setBrush(QColor(0, 0, 0))
            self.innerCircle.setBrush(self.visibleColor)
        if self.name == "o":
            # self.setBrush(QColor(Qt.blue))
            # self.innerCircle.setBrush(QColor(Qt.blue))
            # self.innerCircle.setBrush(QColor(0, 0, 0))
            self.innerCircle.setBrush(self.visibleColor)

        self._debugprint()

    def hoverLeaveEvent(self, event):
        # self.logger.debug("Leaving hover")

        # self.setRect(-6, -6, 12, 12)
        self.setRect(-4, -4, 7, 7)
        self.innerCircle.setRect(-4, -4, 6.5, 6.5)
        if len(self.connectionList) == 0:
            if self.name == "i":
                # self.setBrush(Qt.red)
                # self.setBrush(self.ashColorR)
                # self.innerCircle.setBrush(self.ashColorR)
                # self.innerCircle.setBrush(QColor(0, 0, 0))
                self.innerCircle.setBrush(self.visibleColor)
            if self.name == "o":
                # self.setBrush(self.ashColorR)
                # self.innerCircle.setBrush(self.ashColorB)
                # self.innerCircle.setBrush(QColor(0, 0, 0))
                self.innerCircle.setBrush(self.visibleColor)

        self._debugClear()

    def _debugprint(self):
        self.parent.parent.parent().listV.addItem("This is a PortItem")
        self.parent.parent.parent().listV.addItem("Connections:")
        for c in self.connectionList:
            self.parent.parent.parent().listV.addItem(c.displayName)
        self.parent.parent.parent().listV.addItem(
            "Flipped state (H,V):" + str(self.parent.flippedH) + ", " + str(self.parent.flippedV)
        )
        self.parent.parent.parent().listV.addItem("Side: " + str(self.side))
        self.parent.parent.parent().listV.addItem("createdAtSide: " + str(self.createdAtSide))
        self.parent.parent.parent().listV.addItem("ID: " + str(self.id))
        self.parent.parent.parent().listV.addItem("Block: " + self.parent.displayName)

    def _debugClear(self):
        self.parent.parent.parent().listV.clear()

    def hideCorners(self, connection):
        cor = connection.getCorners()[0]
        cor2 = connection.getCorners()[-1]
        cor.setVisible(False)
        cor2.setVisible(False)

    def showCorners(self, connection):
        cor = connection.getCorners()[0]
        cor2 = connection.getCorners()[-1]
        cor.setVisible(True)
        cor2.setVisible(True)



import dataclasses as _dc

import trnsysGUI.PortItem as _port


@_dc.dataclass
class DirectPortPair:
    trnsysId: int
    fromPort: _port.PortItem  # type: ignore[name-defined]
    toPort: _port.PortItem  # type: ignore[name-defined]
    relativeInputHeight: float
    relativeOutputHeight: float
    isOnLeftSide: bool

    @property
    def relativeInputHeightPercent(self):
        return self._toPercent(self.relativeInputHeight)

    @property
    def relativeOutputHeightPercent(self):
        return self._toPercent(self.relativeOutputHeight)

    @property
    def side(self) -> str:
        return "Left" if self.isOnLeftSide else "Right"

    def setRelativeHeights(self,
                           relativeInputHeight: float,
                           relativeOutputHeight: float,
                           storageTankHeight: float) -> None:
        self.relativeInputHeight = relativeInputHeight
        self.relativeOutputHeight = relativeOutputHeight

        inputPosY = storageTankHeight - storageTankHeight*self.relativeInputHeight
        self.fromPort.setY(inputPosY)

        outputPosY = storageTankHeight - storageTankHeight*self.relativeOutputHeight
        self.toPort.setY(outputPosY)

    @staticmethod
    def _toPercent(relativeHeight: float):
        return int(round(relativeHeight * 100, 2))



# pylint: skip-file
# type: ignore

import typing as _tp

from PyQt5.QtCore import QSize
from PyQt5.QtGui import QPixmap

from trnsysGUI.BlockItem import BlockItem
from trnsysGUI.PortItem import PortItem
import trnsysGUI.images as _img


class WTap_main(BlockItem):
    def __init__(self, trnsysType, parent, **kwargs):
        super(WTap_main, self).__init__(trnsysType, parent, **kwargs)
        self.w = 40
        self.h = 40
        # self.inputs.append(PortItem('i', 0, self))
        self.outputs.append(PortItem("o", 0, self))

        self.exportInitialInput = 0.0

        self.typeNumber = 4

        self.changeSize()

    def _getImageAccessor(self) -> _tp.Optional[_img.ImageAccessor]:
        return _img.W_TAP_MAIN_SVG

    def changeSize(self):
        w = self.w
        h = self.h

        """ Resize block function """
        delta = 20

        # Limit the block size:
        if h < 20:
            h = 20
        if w < 40:
            w = 40
        # center label:
        rect = self.label.boundingRect()
        lw, lh = rect.width(), rect.height()
        lx = (w - lw) / 2
        self.label.setPos(lx, h)

        self.origOutputsPos = [[0, delta]]
        self.outputs[0].setPos(self.origOutputsPos[0][0], self.origOutputsPos[0][1])

        self.updateFlipStateH(self.flippedH)
        self.updateFlipStateV(self.flippedV)

        self.outputs[0].side = (self.rotationN + 2 * self.flippedH) % 4

        return w, h

    def exportBlackBox(self):
        equation = ["T" + self.displayName + "=Tcw"]
        return "success", equation

    def exportMassFlows(self):
        resStr = "Mfr" + self.displayName + " = 1000" + "\n"
        equationNr = 1
        return resStr, equationNr

    def exportParametersFlowSolver(self, descConnLength):
        temp = ""
        for i in self.inputs:
            for c in i.connectionList:
                temp = temp + str(c.trnsysId) + " "
                self.trnsysConn.append(c)

        for o in self.outputs:
            for c in o.connectionList:
                temp = temp + str(c.trnsysId) + " "
                self.trnsysConn.append(c)

        temp += "0 0 "
        temp += str(self.typeNumber)
        temp += " " * (descConnLength - len(temp))
        self.exportConnsString = temp

        f = temp + "!" + str(self.trnsysId) + " : " + str(self.displayName) + "\n"

        return f

    def exportInputsFlowSolver1(self):
        temp1 = "Mfr" + self.displayName
        self.exportInputName = " " + temp1 + " "
        return self.exportInputName, 1



# pylint: skip-file
# type: ignore

import json

from trnsysGUI.diagram import Editor as _de
from trnsysGUI.BlockItem import BlockItem
from trnsysGUI.Connection import Connection
from trnsysGUI.copyGroup import copyGroup


class Encoder(json.JSONEncoder):
    """
    This class encodes the diagram (entire diagram or clipboard)
    obj is passed along to get some of the diagram editor porperties, for instance the id-generator
    TrnsysObj are then stored in a list of dictionaries, saved to a json file.
    Important: There is a slight naming error, since the dict containing all sub dicts (dicts for each block, connection
    and Id-generator) has the key ".__BlockDct__", although there are also other elements in there.
    """

    def default(self, obj):
        """

        Parameters
        ----------
        obj
        :type obj: trnsysGUI.DiagramEditor.DiagramEditor

        Returns
        -------

        """
        logger = obj.logger

        if isinstance(obj, _de.Editor) or isinstance(obj, copyGroup):
            logger.debug("Is diagram or copygroup")

            res = {}
            blockDct = {".__BlockDct__": True}

            for t in obj.trnsysObj:
                if isinstance(t, BlockItem) and not t.isVisible():
                    logger.debug("Invisible block [probably an insideBlock?]" + str(t) + str(t.displayName))
                    continue

                dictName, dct = t.encode()
                blockDct[dictName + str(t.id)] = dct

            idDict = {
                "__idDct__": True,
                "GlobalId": obj.idGen.getID(),
                "trnsysID": obj.idGen.getTrnsysID(),
                "globalConnID": obj.idGen.getConnID(),
            }
            blockDct["IDs"] = idDict

            nameDict = {"__nameDct__": True, "DiagramName": obj.diagramName, "ProjectFolder": obj.projectFolder}
            blockDct["Strings"] = nameDict

            for g in obj.groupList:
                dct = {}
                dct[".__GroupDict__"] = True
                dct["GroupName"] = g.displayName
                dct["Position"] = g.x, g.y
                dct["Size"] = g.w, g.h

                blockDct["..__GroupDct-" + g.displayName] = dct

            for gi in obj.graphicalObj:
                dictName, dct = gi.encode()
                blockDct[dictName + str(gi.id)] = dct

            res["Blocks"] = blockDct

            return res
        else:
            logger.debug("This is a strange object in Encoder" + type(obj))
            # return super().default(obj)



# pylint: skip-file
# type: ignore

import os
import shutil
from pathlib import Path

from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QTreeView, QMenu, QMessageBox, QFileDialog, QInputDialog


class MyQTreeView(QTreeView):
    def __init__(self, model, blockitem):
        """
        Self defined QTreeView that contains operations related to the file explorer
        """
        super(QTreeView, self).__init__()
        self.logger = blockitem.logger
        self.setContextMenuPolicy(True)
        self.customContextMenuRequested.connect(self.contextMenuEvent)
        self.model = model
        self.item = blockitem

        # for i in range(1, self.model.columnCount()-1):
        #     self.logger.debug('Is column %i hidden : %r' % (i, self.isColumnHidden(i)))
        #     self.hideColumn(i)
        #     self.logger.debug('Is column %i hidden : %r' % (i, self.isColumnHidden(i)))

    def mouseDoubleClickEvent(self, event):
        self.logger.debug("Double clicked")
        self.openFile()

    def mousePressEvent(self, event):
        self.clearSelection()
        QTreeView.mousePressEvent(self, event)

    def contextMenuEvent(self, event):
        menu = QMenu()

        # open = menu.addAction("Open")
        # open.triggered.connect(self.openFile)

        load = menu.addAction("Load")
        load.triggered.connect(self.loadFile)

        # editP = menu.addAction("Load profile")
        # editP.triggered.connect(self.loadProfile)

        dele = menu.addAction("Delete")
        dele.triggered.connect(self.delFile)

        menu.exec_(event.globalPos())

    def openFile(self):
        """
        When double click on a row, this method is called to open the double clicked file
        """
        self.logger.debug("Opening file")
        filePath = self.getFilePath()
        try:
            if os.path.isfile(filePath):
                os.startfile(filePath)
        except OSError:
            msg = QMessageBox()
            msg.setText("No application is associated with the specified file for this operation")
            msg.exec_()

    def loadFile(self):
        """
        Loads a file into the project path defined by the user.
        Checks if file already exists and allows user to override or cancel the load.
        """
        filePath = self.getFilePath()
        fileName = QFileDialog.getOpenFileName(self, "Load file")[0]
        simpFileName = os.path.split(fileName)[-1]
        loadPath = os.path.join(filePath, simpFileName)
        self.logger.debug(loadPath)
        if fileName != "":
            self.logger.info("file loaded into %s" % filePath)
            if Path(loadPath).exists():
                qmb = QMessageBox()
                qmb.setText(
                    "Warning: " + "A file with the same name exists already. Do you want to overwrite or cancel?"
                )
                qmb.setStandardButtons(QMessageBox.Save | QMessageBox.Cancel)
                qmb.setDefaultButton(QMessageBox.Cancel)
                ret = qmb.exec()
                if ret == QMessageBox.Save:
                    self.logger.debug("Overwriting")
                    # continue
                else:
                    self.logger.debug("Canceling")
                    return
            shutil.copy(fileName, filePath)
            # self.item.parent().centralWidget.fileList.append(filePath)
            try:
                fileList = self.item.parent.parent().fileList
            except AttributeError:
                fileList = self.item.parent().centralWidget.fileList
            fileList.append(loadPath)
            try:
                self.item.loadedFiles.append(loadPath)
            except AttributeError:
                self.logger.debug("This is the general file browser")
            else:
                self.logger.debug("This is the blockitem file browser")
        else:
            self.logger.info("file loading cancelled")

    def delFile(self):
        """
        Deletes the selected file from the folder
        """
        self.logger.debug("Deleting file")
        filePath = self.rreplace(str(self.getSelectedFile()), "/", "\\", 1)
        if filePath == 0:
            return
        try:
            os.remove(filePath)
            # try:
            #     self.item.parent.parent().fileList.remove(str(filePath))
            # except AttributeError:
            #     try:
            #         self.item.parent().centralWidget.fileList.remove(str(filePath))
            #     except ValueError:
            #         try:
            #             self.item.parent().centralWidget.fileList.remove(str(filePath).replace('/', '\\'))
            #         except:
            #             pass

        except OSError:
            msg = QMessageBox()
            msg.setText("Cannot delete file!")
            msg.exec_()

    def rreplace(self, string, old, new, occurrence):
        li = string.rsplit(old, occurrence)
        return new.join(li)

    # def loadProfile(self):
    #     filePath = self.model.rootPath()
    #     fileName = QFileDialog.getOpenFileName(self, "Load file")[0]
    #     self.logger.debug("filepath: " + filePath)
    #     simpFileName = fileName.split("/ddck/")[-1]
    #     profileName = fileName.split("/")[-1]
    #     self.logger.debug("simpefilename: " + simpFileName)
    #     loadPath = os.path.join(filePath, simpFileName)
    #     self.logger.debug("loadpath: " + loadPath)
    #     if fileName != '':
    #         directory = loadPath.split(profileName)[0]
    #         self.logger.debug("directory: " + directory)
    #         self.logger.info("profile loaded into %s" % filePath)
    #         if Path(loadPath).exists():
    #             qmb = QMessageBox()
    #             qmb.setText("Warning: " +
    #                         "A profile with the same name exists already. Do you want to overwrite or cancel?")
    #             qmb.setStandardButtons(QMessageBox.Save | QMessageBox.Cancel)
    #             qmb.setDefaultButton(QMessageBox.Cancel)
    #             ret = qmb.exec()
    #             if ret == QMessageBox.Save:
    #                 self.logger.debug("Overwriting")
    #                 shutil.copy(fileName, loadPath)
    #                 # continue
    #             else:
    #                 self.logger.debug("Canceling")
    #                 return
    #         else:
    #             if not os.path.exists(directory):
    #                 os.makedirs(directory)
    #             shutil.copy(fileName, loadPath)
    #     else:
    #         self.logger.info("file loading cancelled")

    def getFilePath(self):
        """
        Get the index of the selected file.
        If nothing is selected, get the index of the top file
        """
        index = self.currentIndex()
        return self.model.filePath(index)

    def getSelectedFile(self):
        """
        Gets the selected file, used for delection.
        """
        try:
            index = self.selectedIndexes()[0]
        except IndexError:
            msg = QMessageBox()
            msg.setText("No item at current position!")
            msg.exec_()
            return 0
        else:
            return self.model.filePath(index)

    def getInteger(self):
        i, okPressed = QInputDialog.getInt(self, "Get integer", "Value:", 28, 0, 100, 1)
        if okPressed:
            return i



# pylint: skip-file
# type: ignore

__all__ = ["ImageAccessor"]

import pkgutil as _pu
import logging as _log
import typing as _tp
import abc as _abc
import pathlib as _pl

import PyQt5.QtGui as _qtg
import PyQt5.QtSvg as _qsvg
import PyQt5.QtCore as _qtc

import trnsysGUI

_logger = _log.getLogger("root")


class _DataLoaderBase(_abc.ABC):
    def __init__(self, logger: _log.Logger):
        self._logger = logger

    def loadData(self) -> bytes:
        try:
            data = self._loadDataImpl()
            if not data:
                raise AssertionError("Image data is empty.")
        except Exception as e:
            self._logger.exception(
                "An exception occurred loading image data for '%s'.",
                self.getResourcePath(),
                exc_info=True,
                stack_info=True,
            )
            raise e
        return data

    @_abc.abstractmethod
    def getResourcePath(self) -> str:
        pass

    @_abc.abstractmethod
    def getExtension(self) -> _tp.Optional[str]:
        pass

    @_abc.abstractmethod
    def _loadDataImpl(self) -> bytes:
        pass


class _PackageResourceDataLoader(_DataLoaderBase):
    PATH_PREFIX = "pkg_resource:"

    def __init__(self, packagePath: str, logger: _log.Logger):
        super().__init__(logger)

        self._resourcePath = packagePath

    def getResourcePath(self) -> str:
        return f"{self.PATH_PREFIX}{self._resourcePath}"

    def getExtension(self) -> _tp.Optional[str]:
        parts = self._resourcePath.split(".")

        extension = parts[-1] if parts else None

        return extension

    def _loadDataImpl(self) -> bytes:
        data = _pu.get_data(trnsysGUI.__name__, self._resourcePath)
        return data


class _FileDataLoader(_DataLoaderBase):
    PATH_PREFIX = "file:"

    def __init__(self, filePath: _pl.Path, logger: _log.Logger):
        super().__init__(logger)

        self._absoluteFilePath = filePath.absolute()

    def getResourcePath(self) -> str:
        return f"{self.PATH_PREFIX}{self._absoluteFilePath}"

    def getExtension(self) -> _tp.Optional[str]:
        extension = self._absoluteFilePath.suffix
        if not extension:
            return None

        return extension

    def _loadDataImpl(self) -> bytes:
        return self._absoluteFilePath.read_bytes()


class ImageAccessor:
    def __init__(self, dataLoader: _DataLoaderBase) -> None:
        self._dataLoader = dataLoader

    @staticmethod
    def createForPackageResource(resourcePath: str, logger: _log.Logger = _logger):
        dataLoader = _PackageResourceDataLoader(resourcePath, logger)

        return ImageAccessor(dataLoader)

    @staticmethod
    def createForFile(filePath: _pl.Path, logger: _log.Logger = _logger):
        dataLoader = _FileDataLoader(filePath, logger)

        return ImageAccessor(dataLoader)

    @classmethod
    def createFromResourcePath(cls, resourcePath: str, logger: _log.Logger = _logger) -> "ImageAccessor":
        dataLoader = cls._createDataLoaderFromResourcePath(resourcePath, logger)

        return ImageAccessor(dataLoader)

    @staticmethod
    def _createDataLoaderFromResourcePath(resourcePath: str, logger: _log.Logger) -> _DataLoaderBase:
        if resourcePath.startswith(_PackageResourceDataLoader.PATH_PREFIX):
            path = resourcePath[len(_PackageResourceDataLoader.PATH_PREFIX):]
            return _PackageResourceDataLoader(path, logger)

        if resourcePath.startswith(_FileDataLoader.PATH_PREFIX):
            path = resourcePath[len(_FileDataLoader.PATH_PREFIX):]
            return _FileDataLoader(_pl.Path(path), logger)

        logger.warning("Found legacy resource path %s: assuming it's a package resource.", resourcePath)
        return _PackageResourceDataLoader(resourcePath, logger)

    def getResourcePath(self) -> str:
        return self._dataLoader.getResourcePath()

    def getFileExtension(self):
        return self._dataLoader.getExtension()

    def pixmap(self, *, width: _tp.Optional[int] = None, height: _tp.Optional[int] = None) -> _qtg.QPixmap:
        image = self.image(width=width, height=height)

        return _qtg.QPixmap(image)

    def icon(self) -> _qtg.QIcon:
        pixmap = self.pixmap()

        return _qtg.QIcon(pixmap)

    def image(self, *, width: _tp.Optional[int] = None, height: _tp.Optional[int] = None) -> _qtg.QImage:
        if self.getFileExtension() == "svg":
            return self._svgImage(width=width, height=height)

        imageBytes = self._loadBytes()
        image = _qtg.QImage.fromData(imageBytes)

        size = self._getSize(image.size(), width=width, height=height)
        scaledImage = image.scaled(size)

        return scaledImage

    def _svgImage(self, *, width: int, height: int) -> _qtg.QImage:
        imageBytes = self._loadBytes()
        svgRenderer = _qsvg.QSvgRenderer(imageBytes)

        defaultSize = svgRenderer.defaultSize()
        size = self._getSize(defaultSize, width=width, height=height)

        image = _qtg.QImage(size, _qtg.QImage.Format_ARGB32_Premultiplied)
        image.fill(_qtc.Qt.transparent)
        painter = _qtg.QPainter(image)
        svgRenderer.render(painter)

        return image

    @staticmethod
    def _getSize(defaultSize: _qtc.QSize, *, width: int, height: int) -> _qtc.QSize:
        width = width if width else defaultSize.width()
        height = height if height else defaultSize.height()

        return _qtc.QSize(width, height)

    def _loadBytes(self) -> bytes:
        return self._dataLoader.loadData()



# pylint: skip-file
# type: ignore

from PyQt5.QtCore import QPoint
from PyQt5.QtGui import QPainter
from PyQt5.QtWidgets import QGraphicsView
from trnsysGUI.AirSourceHP import AirSourceHP
from trnsysGUI.BlockItem import BlockItem
from trnsysGUI.Boiler import Boiler
from trnsysGUI.Collector import Collector
from trnsysGUI.Connector import Connector
from trnsysGUI.Control import Control
from trnsysGUI.DeleteBlockCommand import DeleteBlockCommand
from trnsysGUI.ExternalHx import ExternalHx
from trnsysGUI.GenericBlock import GenericBlock
from trnsysGUI.Graphicaltem import GraphicalItem
from trnsysGUI.GroundSourceHx import GroundSourceHx
from trnsysGUI.HPDoubleDual import HPDoubleDual
from trnsysGUI.HeatPump import HeatPump
from trnsysGUI.HeatPumpTwoHx import HeatPumpTwoHx
from trnsysGUI.IceStorage import IceStorage
from trnsysGUI.IceStorageTwoHx import IceStorageTwoHx
from trnsysGUI.MasterControl import MasterControl
from trnsysGUI.PV import PV
from trnsysGUI.PitStorage import PitStorage
from trnsysGUI.Pump import Pump
from trnsysGUI.Radiator import Radiator
from trnsysGUI.StorageTank import StorageTank
from trnsysGUI.TVentil import TVentil
from trnsysGUI.TeePiece import TeePiece
from trnsysGUI.WTap import WTap
from trnsysGUI.WTap_main import WTap_main


class View(QGraphicsView):
    """
    Displays the items from the Scene. Here, the drag and drop from the library to the View is implemented.

    """

    def __init__(self, scene, parent=None):
        QGraphicsView.__init__(self, scene, parent)

        self.logger = parent.logger

        # self.setMinimumSize(self.parent().horizontalLayout.height, 700)
        self.adjustSize()
        # self.setMinimumSize(1300, 700)
        self.parent().diagramScene.viewRect2.setPos(-self.width() / 2, -self.height() / 2)
        # self.parent().diagramScene.viewRect2.setPos(1300, 700)
        # Use aliasing or not:
        self.setRenderHint(QPainter.Antialiasing)

    def dragEnterEvent(self, event):
        if event.mimeData().hasFormat("component/name"):
            event.accept()

    def dragMoveEvent(self, event):
        if event.mimeData().hasFormat("component/name"):
            event.accept()

    def dropEvent(self, event):
        """Here, the dropped icons create BlockItems/GraphicalItems"""
        if event.mimeData().hasFormat("component/name"):
            name = str(event.mimeData().data("component/name"), encoding="utf-8")
            self.logger.debug("name is " + name)
            if name == "StorageTank":
                bl = StorageTank(name, self)
                # c = ConfigStorage(bl, self)
                self.parent().showConfigStorageDlg(bl)
            elif name == "TeePiece":
                bl = TeePiece(name, self)
            elif name == "TVentil":
                bl = TVentil(name, self)
            elif name == "Pump":
                bl = Pump(name, self)
            elif name == "Collector":
                bl = Collector(name, self)
            elif name == "HP":
                bl = HeatPump(name, self)
            elif name == "IceStorage":
                bl = IceStorage(name, self)
            elif name == "PitStorage":
                bl = PitStorage(name, self)
            elif name == "Radiator":
                bl = Radiator(name, self)
            elif name == "WTap":
                bl = WTap(name, self)
            elif name == "WTap_main":
                bl = WTap_main(name, self)
            elif name == "Connector":
                bl = Connector(name, self)
            elif name == "GenericBlock":
                bl = GenericBlock(name, self)
                # c = GenericPortPairDlg(bl, self)
                self.parent().showGenericPortPairDlg(bl)
            elif name == "HPTwoHx":
                bl = HeatPumpTwoHx(name, self)
            elif name == "HPDoubleDual":
                bl = HPDoubleDual(name, self)
            elif name == "Boiler":
                bl = Boiler(name, self)
            elif name == "AirSourceHP":
                bl = AirSourceHP(name, self)
            elif name == "PV":
                bl = PV(name, self)
            elif name == "GroundSourceHx":
                bl = GroundSourceHx(name, self)
            elif name == "ExternalHx":
                bl = ExternalHx(name, self)
            elif name == "IceStorageTwoHx":
                bl = IceStorageTwoHx(name, self)
            elif name == "GraphicalItem":
                bl = GraphicalItem(self)
            elif name == "MasterControl":
                bl = MasterControl(name, self)
            elif name == "Control":
                bl = Control(name, self)
            else:
                bl = BlockItem(name, self)

            snapSize = self.parent().snapSize
            if self.parent().snapGrid:
                qp = QPoint(event.pos().x() - event.pos().x() % snapSize, event.pos().y() - event.pos().y() % snapSize)
                p1 = self.mapToScene(qp)
            else:
                p1 = self.mapToScene(event.pos())

            bl.setPos(p1)
            self.scene().addItem(bl)

            bl.oldPos = bl.scenePos()

            # Debug parent of blockitem
            # print("scene items" + str(self.scene().items()))
            # print("scene parent" + str(self.scene().parent()))
            # print("view children " + str(self.children()))
            # print("view items " + str(self.children()))
            # print("scene children" + str(self.scene().children()))
            # print("bl parentobjects are " + str(bl.parentObject()))
            # print("bl parent Widgets are " + str(bl.parentWidget()))
            # print("bl scene is " + str(bl.scene()))

    def mouseMoveEvent(self, event):
        QGraphicsView.mouseMoveEvent(self, event)
        self.parent().mouseMoveEvent(event)

    def mouseReleaseEvent(self, mouseEvent):
        pass
        # print(str(mouseEvent.pos()))
        #     for ot in self.trnsysObj:
        #     t.setPos(t.pos())
        # #     self.parent().alignYLineItem.setVisible(False)

        super(View, self).mouseReleaseEvent(mouseEvent)

    def wheelEvent(self, event):
        super(View, self).wheelEvent(event)
        # 67108864(dez) = 100000000000000000000000000(bin)
        if int(event.modifiers()) == 67108864:
            if event.angleDelta().y() > 0:
                self.scale(1.2, 1.2)
            else:
                self.scale(0.8, 0.8)

    def mousePressEvent(self, event):
        # for t in self.parent().trnsysObj:
        #     if isinstance(t, BlockItem):
        #         t.alignMode = True
        #         print("Changing alignmentmode")
        # for t in self.parent().trnsysObj:
        #     if isinstance(t, BlockItem):
        #         t.itemChange(t.ItemPositionChange, t.pos())

        super(View, self).mousePressEvent(event)

    def deleteBlockCom(self, bl):
        """
        Pushes the deleteBlockCommand onto the undoStack
        Parameters
        ----------
        bl

        Returns
        -------

        """
        command = DeleteBlockCommand(bl, "Delete block command")
        print("Deleted block")
        self.parent().parent().undoStack.push(command)
